@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012-2015 Tim Ruehsen
- * Copyright(c) 2015-2017 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -25,8 +25,8 @@
  *
  */
 
-#ifndef _LIBWGET_LIBWGET_H
-#define _LIBWGET_LIBWGET_H
+#ifndef WGET_WGET_H
+#define WGET_WGET_H
 
 #include <stddef.h>
 #include <sys/types.h>
@@ -252,7 +252,7 @@ WGET_BEGIN_DECLS
 #define WGET_E_GPG_DISABLED -8 /* GPGME was not enabled at compile time */
 #define WGET_E_GPG_VER_FAIL -9 /* 1 or more non-valid signatures */
 #define WGET_E_GPG_VER_ERR -11 /* Verification failed, GPGME error */
-
+#define WGET_E_XML_PARSE_ERR -12 /* XML parsing failed */
 
 typedef void (*wget_global_get_func_t)(const char *, size_t);
 
@@ -543,6 +543,14 @@ WGETAPI char *
 	wget_vaprintf(const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(1,0);
 WGETAPI char *
 	wget_aprintf(const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(1,2);
+WGETAPI size_t
+	wget_vfprintf(FILE *fp, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(2,0);
+WGETAPI size_t
+	wget_fprintf(FILE *fp, const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(2,3);
+WGETAPI size_t
+	wget_vsnprintf(char *str, size_t size, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(3,0);
+WGETAPI size_t
+	wget_snprintf(char *str, size_t size, const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(3,4);
 
 /*
  * Logger routines
@@ -604,7 +612,9 @@ typedef int (*wget_vector_browse_t)(void *ctx, void *elem);
 typedef int (*wget_vector_destructor_t)(void *elem);
 
 WGETAPI wget_vector_t *
-	wget_vector_create(int max, int off, wget_vector_compare_t cmp) G_GNUC_WGET_MALLOC;
+	wget_vector_create(int max, wget_vector_compare_t cmp) G_GNUC_WGET_MALLOC;
+WGETAPI void
+	wget_vector_set_growth_policy(wget_vector_t *v, int off);
 WGETAPI int
 	wget_vector_find(const wget_vector_t *v, const void *elem) G_GNUC_WGET_NONNULL((2));
 WGETAPI int
@@ -624,11 +634,11 @@ WGETAPI int
 WGETAPI int
 	wget_vector_add_noalloc(wget_vector_t *v, const void *elem) G_GNUC_WGET_NONNULL((2));
 WGETAPI int
-	wget_vector_add_str(wget_vector_t *v, const char *s) G_GNUC_WGET_NONNULL((2));
+	wget_vector_add_str(wget_vector_t *v, const char *s);
 WGETAPI int
-	wget_vector_add_vprintf(wget_vector_t *v, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(2,0) G_GNUC_WGET_NONNULL((2));
+	wget_vector_add_vprintf(wget_vector_t *v, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(2,0);
 WGETAPI int
-	wget_vector_add_printf(wget_vector_t *v, const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(2,3) G_GNUC_WGET_NONNULL((2));
+	wget_vector_add_printf(wget_vector_t *v, const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(2,3);
 WGETAPI int
 	wget_vector_replace(wget_vector_t *v, const void *elem, size_t size, int pos) G_GNUC_WGET_NONNULL((2));
 WGETAPI int
@@ -686,7 +696,7 @@ WGETAPI int
 WGETAPI int
 	wget_hashmap_size(const wget_hashmap_t *h) G_GNUC_WGET_PURE;
 WGETAPI int
-	wget_hashmap_browse(const wget_hashmap_t *h, wget_hashmap_browse_t browse, void *ctx) G_GNUC_WGET_NONNULL((2));
+	wget_hashmap_browse(const wget_hashmap_t *h, wget_hashmap_browse_t browse, void *ctx);
 WGETAPI void
 	wget_hashmap_free(wget_hashmap_t **h);
 WGETAPI void
@@ -757,12 +767,14 @@ WGETAPI void
 	wget_stringmap_setcmpfunc(wget_stringmap_t *h, wget_stringmap_compare_t cmp);
 WGETAPI void
 	wget_stringmap_sethashfunc(wget_stringmap_t *h, wget_stringmap_hash_t hash);
-WGETAPI void
-	wget_stringmap_setloadfactor(wget_stringmap_t *h, float factor);
 WGETAPI void
 	wget_stringmap_set_key_destructor(wget_hashmap_t *h, wget_stringmap_key_destructor_t destructor);
 WGETAPI void
 	wget_stringmap_set_value_destructor(wget_hashmap_t *h, wget_stringmap_value_destructor_t destructor);
+WGETAPI void
+	wget_stringmap_setloadfactor(wget_stringmap_t *h, float factor);
+WGETAPI void
+	wget_stringmap_set_growth_policy(wget_stringmap_t *h, int off);
 
 /*
  * Thread wrapper routines
@@ -810,17 +822,24 @@ typedef struct _wget_decompressor_st wget_decompressor_t;
 typedef int (*wget_decompressor_sink_t)(void *context, const char *data, size_t length);
 typedef int (*wget_decompressor_error_handler_t)(wget_decompressor_t *dc, int err);
 
-enum {
+typedef enum {
+	wget_content_encoding_unknown = -1,
 	wget_content_encoding_identity = 0,
 	wget_content_encoding_gzip = 1,
 	wget_content_encoding_deflate = 2,
-	wget_content_encoding_lzma = 3,
-	wget_content_encoding_bzip2 = 4,
-	wget_content_encoding_brotli = 5
-};
+	wget_content_encoding_xz = 3,
+	wget_content_encoding_lzma = 4,
+	wget_content_encoding_bzip2 = 5,
+	wget_content_encoding_brotli = 6,
+	wget_content_encoding_max = 7
+} wget_content_encoding_type_t;
 
+WGETAPI G_GNUC_WGET_PURE wget_content_encoding_type_t
+	wget_content_encoding_by_name(const char *name);
+WGETAPI G_GNUC_WGET_PURE const char *
+	wget_content_encoding_to_name(wget_content_encoding_type_t type);
 WGETAPI wget_decompressor_t *
-	wget_decompress_open(int encoding, wget_decompressor_sink_t data_sink, void *context);
+	wget_decompress_open(wget_content_encoding_type_t encoding, wget_decompressor_sink_t data_sink, void *context);
 WGETAPI void
 	wget_decompress_close(wget_decompressor_t *dc);
 WGETAPI int
@@ -913,6 +932,9 @@ typedef struct wget_iri_st {
 	 */
 	size_t
 		dirlen;
+	/// size of memory to hold the parsed URI, it contains 0 bytes
+	size_t
+		msize;
 	/// Port number
 	uint16_t
 		port;
@@ -948,6 +970,8 @@ WGETAPI void
 	wget_iri_free_content(wget_iri_t *iri);
 WGETAPI void
 	wget_iri_set_defaultpage(const char *page);
+WGETAPI int
+	wget_iri_set_defaultport(const char *scheme, unsigned short port);
 WGETAPI bool
 	wget_iri_supported(const wget_iri_t *iri) G_GNUC_WGET_PURE G_GNUC_WGET_NONNULL_ALL;
 WGETAPI bool
@@ -964,6 +988,8 @@ WGETAPI int
 	wget_iri_compare(wget_iri_t *iri1, wget_iri_t *iri2) G_GNUC_WGET_PURE;
 WGETAPI char *
 	wget_iri_unescape_inline(char *src) G_GNUC_WGET_NONNULL_ALL;
+WGETAPI char *
+	wget_iri_unescape_url_inline(char *src) G_GNUC_WGET_NONNULL_ALL;
 WGETAPI wget_iri_t *
 	wget_iri_parse(const char *uri, const char *encoding);
 WGETAPI wget_iri_t *
@@ -1656,7 +1682,7 @@ WGETAPI void
 
 typedef void (*wget_xml_callback_t)(void *, int, const char *, const char *, const char *, size_t, size_t);
 
-WGETAPI void
+WGETAPI int
 	wget_xml_parse_buffer(
 		const char *buf,
 		wget_xml_callback_t callback,
@@ -2198,7 +2224,7 @@ typedef struct _wget_hash_hd_st wget_hash_hd_t;
  * \brief Enumeration of different hash digest algorithms
  */
 typedef enum {
-	WGET_DIGTYPE_UNKNOWN, /**< Indicates 'Unknown hash algorithm', returned by wget_hash_get_algorithm() */
+	WGET_DIGTYPE_UNKNOWN = 0, /**< Indicates 'Unknown hash algorithm', returned by wget_hash_get_algorithm() */
 	WGET_DIGTYPE_MD5,     /**< Type 'MD5' digest */
 	WGET_DIGTYPE_SHA1,    /**< Type SHA1 digest */
 	WGET_DIGTYPE_RMD160,  /**< Type RMD160 digest */
@@ -2206,7 +2232,8 @@ typedef enum {
 	WGET_DIGTYPE_SHA256,  /**< Type 'SHA256' digest */
 	WGET_DIGTYPE_SHA384,  /**< Type 'SHA384' digest */
 	WGET_DIGTYPE_SHA512,  /**< Type 'SHA512' digest */
-	WGET_DIGTYPE_SHA224   /**< Type 'SHA224' digest */
+	WGET_DIGTYPE_SHA224,  /**< Type 'SHA224' digest */
+	WGET_DIGTYPE_MAX      /**< Number of digest types */
 } wget_digest_algorithm_t;
 
 WGETAPI wget_digest_algorithm_t
@@ -2282,7 +2309,7 @@ typedef struct {
 } wget_metalink_t;
 
 WGETAPI wget_metalink_t
-	*wget_metalink_parse(const char *xml) G_GNUC_WGET_NONNULL((1));
+	*wget_metalink_parse(const char *xml);
 WGETAPI void
 	wget_metalink_free(wget_metalink_t **metalink);
 WGETAPI void
@@ -2318,9 +2345,9 @@ WGETAPI void
 	wget_bar_free(wget_bar_t **bar);
 WGETAPI void
 	wget_bar_print(wget_bar_t *bar, int slot, const char *s);
-WGETAPI ssize_t
+WGETAPI void
 	wget_bar_vprintf(wget_bar_t *bar, int slot, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(3,0) G_GNUC_WGET_NONNULL_ALL;
-WGETAPI ssize_t
+WGETAPI void
 	wget_bar_printf(wget_bar_t *bar, int slot, const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(3,4) G_GNUC_WGET_NONNULL_ALL;
 WGETAPI void
 	wget_bar_slot_begin(wget_bar_t *bar, int slot, const char *filename, ssize_t filesize) G_GNUC_WGET_NONNULL_ALL;
@@ -2588,17 +2615,15 @@ struct wget_plugin_vtable
 
 typedef enum {
 	WGET_STATS_TYPE_DNS = 0,
-	WGET_STATS_TYPE_TLS = 1,
+	WGET_STATS_TYPE_OCSP = 1,
 	WGET_STATS_TYPE_SERVER = 2,
-	WGET_STATS_TYPE_OCSP = 3,
-	WGET_STATS_TYPE_SITE = 4
+	WGET_STATS_TYPE_SITE = 3,
+	WGET_STATS_TYPE_TLS = 4,
 } wget_stats_type_t;
 
 typedef enum {
 	WGET_STATS_FORMAT_HUMAN = 0,
 	WGET_STATS_FORMAT_CSV = 1,
-	WGET_STATS_FORMAT_JSON = 2,
-	WGET_STATS_FORMAT_TREE = 3
 } wget_stats_format_t;
 
 typedef enum {
@@ -2617,7 +2642,7 @@ typedef enum {
 	WGET_STATS_TLS_SECS = 5,
 	WGET_STATS_TLS_CON = 6,
 	WGET_STATS_TLS_RESUMED = 7,
-	WGET_STATS_TLS_TCP_PROTO = 8,
+	WGET_STATS_TLS_HTTP_PROTO = 8,
 	WGET_STATS_TLS_CERT_CHAIN_SIZE = 9
 } wget_tls_stats_t;
 
@@ -2642,11 +2667,12 @@ typedef enum {
 	WGET_STATS_OCSP_HOSTNAME = 0,
 	WGET_STATS_OCSP_VALID = 1,
 	WGET_STATS_OCSP_REVOKED = 2,
-	WGET_STATS_OCSP_IGNORED = 3
+	WGET_STATS_OCSP_IGNORED = 3,
+	WGET_STATS_OCSP_STAPLING = 4
 } wget_ocsp_stats_t;
 
 typedef void
-	(*wget_stats_callback_t)(wget_stats_type_t type, const void *stats);
+	(*wget_stats_callback_t)(const void *stats);
 
 WGETAPI void
 	wget_tcp_set_stats_dns(wget_stats_callback_t fn);
@@ -2666,25 +2692,25 @@ WGETAPI void
 WGETAPI const void *
 	wget_tcp_get_stats_server(wget_server_stats_t type, const void *stats);
 
+WGETAPI void
+	wget_tcp_set_stats_site(wget_stats_callback_t fn);
+
 WGETAPI void
 	wget_tcp_set_stats_ocsp(wget_stats_callback_t fn);
 
 WGETAPI const void *
 	wget_tcp_get_stats_ocsp(wget_ocsp_stats_t type, const void *stats);
 
-WGETAPI void
-	wget_tcp_set_stats_site(bool stats_site_switch);
-
 WGETAPI void
 	host_ips_free(void);
 
 WGET_END_DECLS
 
-#endif /* _LIBWGET_LIBWGET_H */
-
 /*
  * Regex Types
  */
 
 #define WGET_REGEX_TYPE_POSIX 0
 #define WGET_REGEX_TYPE_PCRE 1
+
+#endif /* WGET_WGET_H */
