@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -32,14 +32,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <ctype.h>
 #include <time.h>
 #include <errno.h>
 #ifdef WITH_LIBPSL
-#	include <libpsl.h>
-#  define _U
-#else
-#  define _U G_GNUC_WGET_UNUSED
+#  include <libpsl.h>
 #endif
 
 #include <c-ctype.h>
@@ -47,6 +45,11 @@
 #include <wget.h>
 #include "private.h"
 
+#ifdef WITH_LIBPSL
+#  undef G_GNUC_WGET_UNUSED
+#  define G_GNUC_WGET_UNUSED
+#endif
+
 struct _wget_cookie_st {
 	const char *
 		name;
@@ -92,7 +95,7 @@ struct _wget_cookie_db_st {
 
 // by this kind of sorting, we can easily see if a domain matches or not (match = supercookie !)
 
-int wget_cookie_db_load_psl(wget_cookie_db_t *cookie_db _U, const char *fname _U)
+int wget_cookie_db_load_psl(G_GNUC_WGET_UNUSED wget_cookie_db_t *cookie_db, G_GNUC_WGET_UNUSED const char *fname)
 {
 #ifdef WITH_LIBPSL
 	if (!cookie_db)
@@ -584,7 +587,7 @@ void wget_cookie_normalize_cookies(const wget_iri_t *iri, const wget_vector_t *c
 //	wget_thread_mutex_unlock(&_cookies_mutex);
 }
 
-int wget_cookie_check_psl(const wget_cookie_db_t *cookie_db _U, const wget_cookie_t *cookie _U)
+int wget_cookie_check_psl(G_GNUC_WGET_UNUSED const wget_cookie_db_t *cookie_db, G_GNUC_WGET_UNUSED const wget_cookie_t *cookie)
 {
 //	wget_thread_mutex_lock(&_cookies_mutex);
 
@@ -708,7 +711,7 @@ char *wget_cookie_create_request_header(wget_cookie_db_t *cookie_db, const wget_
 		debug_printf("found %s=%s\n", cookie->name, cookie->value);
 
 		if (!cookies)
-			cookies = wget_vector_create(16, -2, (wget_vector_compare_t)_compare_cookie2);
+			cookies = wget_vector_create(16, (wget_vector_compare_t)_compare_cookie2);
 
 		// collect matching cookies (just pointers, no allocation)
 		wget_vector_add_noalloc(cookies, cookie);
@@ -747,7 +750,7 @@ wget_cookie_db_t *wget_cookie_db_init(wget_cookie_db_t *cookie_db)
 		cookie_db = xmalloc(sizeof(wget_cookie_db_t));
 
 	memset(cookie_db, 0, sizeof(*cookie_db));
-	cookie_db->cookies = wget_vector_create(32, -2, (wget_vector_compare_t)_compare_cookie);
+	cookie_db->cookies = wget_vector_create(32, (wget_vector_compare_t)_compare_cookie);
 	wget_vector_set_destructor(cookie_db->cookies, (wget_vector_destructor_t)wget_cookie_deinit);
 	wget_thread_mutex_init(&cookie_db->mutex);
 #ifdef WITH_LIBPSL
@@ -920,13 +923,13 @@ static int _cookie_db_save(wget_cookie_db_t *cookie_db, FILE *fp)
 			} else if (!cookie_db->keep_session_cookies)
 				continue;
 
-			fprintf(fp, "%s%s%s\t%s\t%s\t%s\t%"PRId64"\t%s\t%s\n",
+			wget_fprintf(fp, "%s%s%s\t%s\t%s\t%s\t%lld\t%s\t%s\n",
 				cookie->http_only ? "#HttpOnly_" : "",
 				cookie->domain_dot ? "." : "", // compatibility, irrelevant since RFC 6562
 				cookie->domain,
 				cookie->host_only ? "FALSE" : "TRUE",
 				cookie->path, cookie->secure_only ? "TRUE" : "FALSE",
-				(int64_t)cookie->expires,
+				(long long)cookie->expires,
 				cookie->name, cookie->value);
 
 			if (ferror(fp))
