@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -151,6 +151,16 @@ static int _body_callback(wget_http_response_t *resp, void *user_data G_GNUC_WGE
 	return 0;
 }
 
+/*
+ * https://tools.ietf.org/html/rfc7230#section-3: Message Format
+ * https://tools.ietf.org/html/rfc7230#section-3.1: Start Line
+ * https://tools.ietf.org/html/rfc7230#section-3.1.1: Request Line
+ * https://tools.ietf.org/html/rfc7230#section-5.3: Request Target
+ * https://tools.ietf.org/html/rfc7230#section-5.3.1: Origin Form
+ * https://tools.ietf.org/html/rfc7230#section-2.7.3: http and https URI Normalization and Comparison
+ *   Characters other than those in the "reserved" set are equivalent to their
+ *   percent-encoded octets: the normal form is to not encode them (see Sections 2.1 and 2.2 of [RFC3986]).
+ */
 wget_http_request_t *wget_http_create_request(const wget_iri_t *iri, const char *method)
 {
 	wget_http_request_t *req = xcalloc(1, sizeof(wget_http_request_t));
@@ -162,7 +172,7 @@ wget_http_request_t *wget_http_create_request(const wget_iri_t *iri, const char
 	wget_strscpy(req->method, method, sizeof(req->method));
 	wget_iri_get_escaped_resource(iri, &req->esc_resource);
 	wget_iri_get_escaped_host(iri, &req->esc_host);
-	req->headers = wget_vector_create(8, 8, NULL);
+	req->headers = wget_vector_create(8, NULL);
 	wget_vector_set_destructor(req->headers, (wget_vector_destructor_t)wget_http_free_param);
 
 	wget_http_add_header(req, "Host", req->esc_host.data);
@@ -323,7 +333,7 @@ void wget_http_add_credentials(wget_http_request_t *req, wget_http_challenge_t *
 
 		if (!wget_strcmp(algorithm, "MD5-sess") || !wget_strcmp(algorithm, "SHA-256-sess")) {
 			// A1BUF = H( H(user ":" realm ":" password) ":" nonce ":" cnonce )
-			snprintf(cnonce, sizeof(cnonce), "%08x", (unsigned) wget_random()); // create random hex string
+			wget_snprintf(cnonce, sizeof(cnonce), "%08x", (unsigned) wget_random()); // create random hex string
 			wget_hash_printf_hex(hashtype, a1buf, sizeof(a1buf), "%s:%s:%s", a1buf, nonce, cnonce);
 		}
 
@@ -333,7 +343,7 @@ void wget_http_add_credentials(wget_http_request_t *req, wget_http_challenge_t *
 		if (!wget_strcmp(qop, "auth") || !wget_strcmp(qop, "auth-int")) {
 			// RFC 2617 Digest Access Authentication
 			if (!*cnonce)
-				snprintf(cnonce, sizeof(cnonce), "%08x", (unsigned) wget_random()); // create random hex string
+				wget_snprintf(cnonce, sizeof(cnonce), "%08x", (unsigned) wget_random()); // create random hex string
 
 			// RESPONSE_DIGEST = H(A1BUF ":" nonce ":" nc ":" cnonce ":" qop ": " A2BUF)
 			wget_hash_printf_hex(hashtype, response_digest, sizeof(response_digest),
@@ -680,7 +690,7 @@ static void _server_stats_add(wget_http_connection_t *conn, wget_http_response_t
 		stats.hsts = resp ? (resp->hsts ? 1 : 0) : -1;
 		stats.csp = resp ? (resp->csp ? 1 : 0) : -1;
 
-		stats_callback(WGET_STATS_TYPE_SERVER, &stats);
+		stats_callback(&stats);
 		host_add(hostp);
 	} else
 		_free_host_entry(hostp);
@@ -771,11 +781,11 @@ int wget_http_open(wget_http_connection_t **_conn, const wget_iri_t *iri)
 				return WGET_E_INVALID;
 			}
 
-			conn->received_http2_responses = wget_vector_create(16, -2, NULL);
+			conn->received_http2_responses = wget_vector_create(16, NULL);
 		} else
-			conn->pending_requests = wget_vector_create(16, -2, NULL);
+			conn->pending_requests = wget_vector_create(16, NULL);
 #else
-		conn->pending_requests = wget_vector_create(16, -2, NULL);
+		conn->pending_requests = wget_vector_create(16, NULL);
 #endif
 	} else {
 		if (stats_callback && (rc == WGET_E_CERTIFICATE))
@@ -911,7 +921,7 @@ ssize_t wget_http_request_to_buffer(wget_http_request_t *req, wget_buffer_t *buf
 	char have_content_length = 0;
 	char check_content_length = req->body && req->body_length;
 
-//	buffer_sprintf(buf, "%s /%s HTTP/1.1\r\nHost: %s", req->method, req->esc_resource.data ? req->esc_resource.data : "",);
+//	wget_buffer_sprintf(buf, "%s /%s HTTP/1.1\r\nHost: %s", req->method, req->esc_resource.data ? req->esc_resource.data : "",);
 
 	wget_buffer_strcpy(buf, req->method);
 	wget_buffer_memcat(buf, " ", 1);
@@ -955,6 +965,24 @@ ssize_t wget_http_request_to_buffer(wget_http_request_t *req, wget_buffer_t *buf
 	return buf->length;
 }
 
+static void _fix_broken_server_encoding(wget_http_response_t *resp)
+{
+	// a workaround for broken server configurations
+	// see https://mail-archives.apache.org/mod_mbox/httpd-dev/200207.mbox/<3D2D4E76.4010502@talex.com.pl>
+	if (resp->content_encoding == wget_content_encoding_gzip) {
+		const char *ext;
+		if (!wget_strcasecmp_ascii(resp->content_type, "application/x-gzip")
+			|| !wget_strcasecmp_ascii(resp->content_type, "application/gzip")
+			|| !wget_strcasecmp_ascii(resp->content_type, "application/gunzip")
+			|| ((ext = strrchr(resp->req->esc_resource.data, '.'))
+			&& (!wget_strcasecmp_ascii(ext, ".gz") || !wget_strcasecmp_ascii(ext, ".tgz"))))
+		{
+			debug_printf("Broken server configuration gzip workaround triggered\n");
+			resp->content_encoding =  wget_content_encoding_identity;
+		}
+	}
+}
+
 wget_http_response_t *wget_http_get_response_cb(wget_http_connection_t *conn)
 {
 	size_t bufsize, body_len = 0, body_size = 0;
@@ -1007,25 +1035,17 @@ wget_http_response_t *wget_http_get_response_cb(wget_http_connection_t *conn)
 		}
 
 		resp = wget_vector_get(conn->received_http2_responses, 0); // should use double linked lists here
+
+		if (stats_callback)
+			_server_stats_add(conn, resp);
+
 		if (resp) {
 			debug_printf("  ##  response status %d\n", resp->code);
 			wget_vector_remove_nofree(conn->received_http2_responses, 0);
 
-			// a workaround for broken server configurations
-			// see https://mail-archives.apache.org/mod_mbox/httpd-dev/200207.mbox/<3D2D4E76.4010502@talex.com.pl>
-			if (resp->content_encoding == wget_content_encoding_gzip
-				&& (!wget_strcasecmp_ascii(resp->content_type, "application/x-gzip")
-				   || !wget_strcasecmp_ascii(resp->content_type, "application/gzip")
-				   || !wget_strcasecmp_ascii(resp->content_type, "application/gunzip")))
-			{
-				debug_printf("Broken server configuration gzip workaround triggered\n");
-				resp->content_encoding =  wget_content_encoding_identity;
-			}
+			_fix_broken_server_encoding(resp);
 		}
 
-		if (stats_callback)
-			_server_stats_add(conn, resp);
-
 		return resp;
 	}
 #endif
@@ -1092,6 +1112,8 @@ wget_http_response_t *wget_http_get_response_cb(wget_http_connection_t *conn)
 			if (req && !wget_strcasecmp_ascii(req->method, "HEAD"))
 				goto cleanup; // a HEAD response won't have a body
 
+			_fix_broken_server_encoding(resp);
+
 			p += 4; // skip \r\n\r\n to point to body
 			break;
 		}
@@ -1227,7 +1249,8 @@ wget_http_response_t *wget_http_get_response_cb(wget_http_connection_t *conn)
 			}
 
 			// check for pointer overflow
-			if (end > end + chunk_size || end >= end + chunk_size + 2) {
+			if (chunk_size > SIZE_MAX/2 - 2) {
+//			if (end > end + chunk_size || end >= end + chunk_size + 2) {
 				error_printf(_("Chunk size overflow: %lX\n"), chunk_size);
 				goto cleanup;
 			}
@@ -1374,7 +1397,7 @@ static wget_vector_t *_parse_proxies(const char *proxy, const char *encoding)
 			iri = wget_iri_parse (host, encoding);
 			if (iri) {
 				if (!proxies) {
-					proxies = wget_vector_create(8, -2, NULL);
+					proxies = wget_vector_create(8, NULL);
 					wget_vector_set_destructor(proxies, (wget_vector_destructor_t)wget_iri_free_content);
 				}
 				wget_vector_add_noalloc(proxies, iri);
@@ -1393,7 +1416,7 @@ static wget_vector_t *_parse_no_proxies(const char *no_proxy, const char *encodi
 	wget_vector_t *proxies;
 	const char *s, *p;
 
-	proxies = wget_vector_create(8, -2, NULL);
+	proxies = wget_vector_create(8, NULL);
 
 	for (s = p = no_proxy; *p; s = p + 1) {
 		while (c_isspace(*s) && s < p) s++;
@@ -1495,6 +1518,15 @@ void wget_tcp_set_stats_server(wget_stats_callback_t fn)
 	stats_callback = fn;
 }
 
+/**
+ * \param[in] fn A `wget_stats_callback_t` callback function used to collect Site Host statistics
+ *
+ * Set callback function to be called once Response statistics for a host are collected
+ */
+void wget_tcp_set_stats_site(G_GNUC_WGET_UNUSED wget_stats_callback_t fn)
+{
+}
+
 /**
  * \param[in] type A `wget_server_stats_t` constant representing Server statistical info to return
  * \param[in] _stats An internal  pointer sent to callback function
