@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -76,7 +76,7 @@ static size_t
 
 const char
 	* const wget_iri_schemes[] = { "http", "https" };
-static const uint16_t
+static uint16_t
 	iri_ports[]   = { 80, 443 }; // default port numbers for the above schemes
 
 /* \cond _hide_internal_symbols */
@@ -206,11 +206,37 @@ bool wget_iri_isunreserved_path(char c)
 	return c > 32 && c < 127 && (c_isalnum(c) || _iri_isunreserved(c) || c == '/');
 }
 
-static _GL_INLINE unsigned char G_GNUC_WGET_CONST _unhex(unsigned char c)
+static unsigned char G_GNUC_WGET_CONST _unhex(unsigned char c)
 {
 	return c <= '9' ? c - '0' : (c <= 'F' ? c - 'A' + 10 : c - 'a' + 10);
 }
 
+static char *_iri_unescape_inline(char *src, int ctype)
+{
+	char *ret = NULL;
+	unsigned char *s = (unsigned char *)src; // just a helper to avoid casting a lot
+	unsigned char *d = s;
+
+	while (*s) {
+		if (*s == '%') {
+			if (c_isxdigit(s[1]) && c_isxdigit(s[2])) {
+				unsigned char c = (unsigned char) (_unhex(s[1]) << 4) | _unhex(s[2]);
+				if (!ctype || (!(iri_ctype[(unsigned char)(c)] & ctype) && c != '%')) {
+					*d++ = c;
+					s += 3;
+					ret = src;
+					continue;
+				}
+			}
+		}
+
+		*d++ = *s++;
+	}
+	*d = 0;
+
+	return ret;
+}
+
 /**
  * \param[in] src A string
  * \return A pointer to \p src, after the transformation is done
@@ -223,25 +249,25 @@ static _GL_INLINE unsigned char G_GNUC_WGET_CONST _unhex(unsigned char c)
  */
 char *wget_iri_unescape_inline(char *src)
 {
-	char *ret = NULL;
-	unsigned char *s = (unsigned char *)src; // just a helper to avoid casting a lot
-	unsigned char *d = s;
+	return _iri_unescape_inline(src, 0);
+}
 
-	while (*s) {
-		if (*s == '%') {
-			if (c_isxdigit(s[1]) && c_isxdigit(s[2])) {
-				*d++ = (unsigned char) (_unhex(s[1]) << 4) | _unhex(s[2]);
-				s += 3;
-				ret = src;
-				continue;
-			}
-		}
-
-		*d++ = *s++;
-	}
-	*d = 0;
-
-	return ret;
+/**
+ * \param[in] src A string
+ * \return A pointer to \p src, after the transformation is done
+ *
+ * Unescape a string except escaped generic delimiters (and escaped '%'.
+ * The percent-encoded characters (`%XX`) are converted back to their original form.
+ *
+ * This variant of unescaping is helpful before an URL is being parsed, so that
+ * the parser recognizes e.g. 'http%3A//' as relative URL (path) and not as a scheme.
+ *
+ * **The transformation is done inline**, so `src` will be modified after this function returns.
+ * If no characters were unescaped, the string is left untouched.
+ */
+char *wget_iri_unescape_url_inline(char *src)
+{
+	return _iri_unescape_inline(src, IRI_CTYPE_GENDELIM);
 }
 
 /**
@@ -306,8 +332,8 @@ wget_iri_t *wget_iri_parse(const char *url, const char *encoding)
 {
 	wget_iri_t *iri;
 	char *p, *s, *authority, c;
-	size_t slen;
-	int maybe_scheme;
+	size_t slen, extra;
+	int have_scheme;
 
 	if (!url)
 		return NULL;
@@ -346,56 +372,62 @@ wget_iri_t *wget_iri_parse(const char *url, const char *encoding)
 		}
 	}
 */
+
+	if (c_isalpha(*url)) {
+		const char *x;
+		have_scheme = 1;
+
+		for (x = url; *x && _iri_isscheme(*x); x++)
+			;
+
+		if (*x != ':' || c_isdigit(x[1]))
+			have_scheme = 0; // not a scheme
+	} else
+		have_scheme = 0;
+
 	// just use one block of memory for all parsed URI parts
 	slen = strlen(url);
-	iri = xmalloc(sizeof(wget_iri_t) + slen * 2 + 2);
+	extra = have_scheme ? 0 : sizeof("http://") - 1; // extra space for http://
+
+	iri = xmalloc(sizeof(wget_iri_t) + (slen + extra + 1) * 2);
 	memset(iri, 0, sizeof(wget_iri_t));
-	iri->uri = memcpy(((char *)iri) + sizeof(wget_iri_t), url, slen + 1);
-	s = memcpy((char *)iri->uri + slen + 1, url, slen + 1);
 
-//	if (url_allocated)
-//		xfree(url);
-
-	p = s;
-	if (c_isalpha(*p)) {
-		maybe_scheme = 1;
-		while (*s && !_iri_isgendelim(*s)) {
-			if (maybe_scheme && !_iri_isscheme(*s))
-				maybe_scheme = 0;
-			s++;
-		}
-	} else
-		maybe_scheme = 0;
-
-//	if (maybe_scheme && (*s == ':' && (s[1] == '/' || s[1] == 0))) {
-	if (maybe_scheme && (*s == ':' && !c_isdigit(s[1]))) {
-		// found a scheme
+	if (have_scheme) {
+		iri->msize = slen + 1;
+		iri->uri = memcpy(((char *)iri) + sizeof(wget_iri_t), url, iri->msize);
+		iri->scheme = s = memcpy((char *)iri->uri + iri->msize, url, iri->msize);
+		s = strchr(s, ':'); // we know there is a :
 		*s++ = 0;
 
+		wget_iri_unescape_inline((char *)iri->scheme); // percent unescape
+		wget_strtolower((char *)iri->scheme); // convert scheme to lowercase
+
 		// find the scheme in our static list of supported schemes
 		// for later comparisons we compare pointers (avoiding strcasecmp())
-		iri->scheme = p;
-		wget_iri_unescape_inline((char *)iri->scheme);
-
 		for (unsigned it = 0; it < countof(wget_iri_schemes); it++) {
-			if (!wget_strcasecmp_ascii(wget_iri_schemes[it], p)) {
+			if (!strcmp(wget_iri_schemes[it], iri->scheme)) {
 				iri->scheme = wget_iri_schemes[it];
 				iri->port = iri_ports[it];
 				break;
 			}
 		}
-
-		if (iri->scheme == p) {
-			// convert scheme to lowercase
-			wget_strtolower((char *)iri->scheme);
-		}
-
 	} else {
+		// add http:// scheme to url
+		iri->uri = memcpy(((char *)iri) + sizeof(wget_iri_t), "http://", extra);
+		memcpy(((char *)iri) + sizeof(wget_iri_t) + extra, url, slen + 1);
+		iri->msize = slen + 1 + extra;
+		s = memcpy((char *)iri->uri + iri->msize, "http://", extra);
+		memcpy((char *)iri->uri + iri->msize + extra, url, slen + 1);
+		s[extra - 3] = 0;
+		s += extra;
+
 		iri->scheme = WGET_IRI_SCHEME_HTTP;
-		iri->port = 80;
-		s = p; // rewind
+		iri->port = iri_ports[0];
 	}
 
+//	if (url_allocated)
+//		xfree(url);
+
 	// this is true for http, https, ftp, file (accept any number of /, like most browsers)
 	while (*s == '/')
 		s++;
@@ -549,11 +581,15 @@ wget_iri_t *wget_iri_parse(const char *url, const char *encoding)
  */
 wget_iri_t *wget_iri_clone(const wget_iri_t *iri)
 {
-	if (!iri)
+	if (!iri || !iri->uri)
 		return NULL;
 
-	size_t slen = iri->uri ? strlen(iri->uri) : 0;
-	wget_iri_t *clone = wget_memdup(iri, sizeof(wget_iri_t) + slen * 2 + 2);
+	size_t slen = strlen(iri->uri);
+	wget_iri_t *clone = wget_malloc(sizeof(wget_iri_t) + (slen + 1) + iri->msize);
+	memcpy(clone, iri, sizeof(wget_iri_t));
+	clone->uri = memcpy(((char *)clone) + sizeof(wget_iri_t), iri->uri, slen + 1);
+	memcpy((char *)clone->uri + slen + 1, (char *)iri->uri + slen + 1, iri->msize);
+	clone->uri_allocated = 0;
 
 	clone->connection_part = wget_strdup(iri->connection_part);
 
@@ -563,10 +599,6 @@ wget_iri_t *wget_iri_clone(const wget_iri_t *iri)
 	else
 		clone->host = iri->host ? (char *)clone + (size_t) (iri->host - (const char *)iri) : NULL;
 
-	if (iri->uri_allocated)
-		clone->uri = wget_strdup(iri->uri);
-	else
-		clone->uri = iri->uri ? (char *)clone + (size_t) (iri->uri - (const char *)iri) : NULL;
 	clone->display = iri->display ? (char *)clone + (size_t) (iri->display - (const char *)iri): NULL;
 	// not adjust scheme, it is a pointer to a static string
 	clone->userinfo = iri->userinfo ? (char *)clone + (size_t) (iri->userinfo - (const char *)iri): NULL;
@@ -1016,10 +1048,14 @@ const char *wget_iri_get_escaped_resource(const wget_iri_t *iri, wget_buffer_t *
 	// Do not actually escape the query field. This part of the URL *MAY*
 	// contain reserved characters which should be passed on as-is and without
 	// escaping them. This is according to the rules laid out in RFC 2616 and
-	// RFC 7230
+	// RFC 7230. But we have to replace spaces in any case.
 	if (iri->query) {
 		wget_buffer_memcat(buf, "?", 1);
-		wget_buffer_strcat(buf, iri->query);
+		for (const char *p = iri->query; *p; p++)
+			if (*p == ' ')
+				wget_buffer_memcat(buf, "%20", 3);
+			else
+				wget_buffer_memcat(buf, p, 1);
 	}
 
 	return buf->data;
@@ -1214,6 +1250,25 @@ void wget_iri_set_defaultpage(const char *page)
 	default_page_length = default_page ? strlen(default_page) : 0;
 }
 
+/**
+ * \param scheme The scheme for the new default port
+ * \param port The new default port value for the given scheme
+ * \return 0: success  -1: Unknown scheme
+ *
+ * Set the default \p port for the given \p scheme.
+ */
+int wget_iri_set_defaultport(const char *scheme, unsigned short port)
+{
+	for (unsigned it = 0; it < countof(wget_iri_schemes); it++) {
+		if (!wget_strcasecmp_ascii(wget_iri_schemes[it], scheme)) {
+			iri_ports[it] = port;
+			return 0;
+		}
+	}
+
+	return -1;
+}
+
 /**
  * \param[in] iri An IRI
  * \param[in] scheme A scheme, such as `http` or `https`.
@@ -1233,7 +1288,7 @@ const char *wget_iri_set_scheme(wget_iri_t *iri, const char *scheme)
 	for (int index = 0; wget_iri_schemes[index]; index++) {
 		if (!wget_strcasecmp_ascii(wget_iri_schemes[index], scheme)) {
 			iri->scheme = wget_iri_schemes[index];
-			// If the IRI is using a port other than the default, keep it untouched
+			// If the IRI is using the default port, also change it
 			if (!iri->port_given)
 				iri->port = iri_ports[index];
 			break;
@@ -1243,9 +1298,10 @@ const char *wget_iri_set_scheme(wget_iri_t *iri, const char *scheme)
 	// Rewrite the URI if scheme has changed
 	if (old_scheme != iri->scheme) {
 		size_t old_scheme_len = strlen(old_scheme);
+
 		if (strncmp(iri->uri, old_scheme, old_scheme_len) == 0) {
 			if (strncmp(iri->uri + old_scheme_len, "://", 3) == 0) {
-				char *new_uri = wget_aprintf("%s://%s",  iri->scheme, iri->uri + old_scheme_len + 3);
+				char *new_uri = wget_aprintf("%s%s",  iri->scheme, iri->uri + old_scheme_len);
 				if (iri->uri_allocated)
 					xfree(iri->uri);
 				iri->uri = new_uri;
