@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -230,24 +230,14 @@ static struct addrinfo *_wget_sort_preferred(struct addrinfo *addrinfo, int pref
 	}
 }
 
-#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
-// for fuzzing and 'make check' with fuzz corpora we don't want real DNS access
-static int _wget_tcp_resolve(
-	G_GNUC_WGET_UNUSED wget_tcp_t *tcp,
-	G_GNUC_WGET_UNUSED const char *host,
-	G_GNUC_WGET_UNUSED uint16_t port,
-	G_GNUC_WGET_UNUSED struct addrinfo **out_addr)
-{
-	return EAI_NODATA;
-}
-#elif defined HAVE_GETADDRINFO_A
+#if defined HAVE_GETADDRINFO_A && ! defined FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 static int _wget_tcp_resolve(wget_tcp_t *tcp, const char *host, uint16_t port, struct addrinfo **out_addr)
 {
 	int err;
 	char s_port[NI_MAXSERV];
 
 	if (port) {
-		snprintf(s_port, sizeof(s_port), "%hu", port);
+		wget_snprintf(s_port, sizeof(s_port), "%hu", port);
 		debug_printf("resolving %s:%s...\n", host ? host : "", s_port);
 	} else {
 		debug_printf("resolving %s...\n", host);
@@ -284,6 +274,9 @@ static int _wget_tcp_resolve(wget_tcp_t *tcp, const char *host, uint16_t port, s
 		// timeout occurred, do we need to cancel ?
 		int cancel_err = gai_cancel(&addr);
 		debug_printf("gai_cancel = %s\n", gai_strerror(cancel_err));
+	} else if (err == EAI_ALLDONE) {
+		debug_printf("gai_suspend(): %s, accept as OK\n", gai_strerror(err));
+		err = 0;
 	} else if (err)
 		debug_printf("gai_suspend() failed (%d): %s\n", err, gai_strerror(err));
 
@@ -312,7 +305,7 @@ static int _wget_tcp_resolve(wget_tcp_t *tcp, const char *host, uint16_t port, s
 	if (port) {
 		char s_port[NI_MAXSERV];
 
-		snprintf(s_port, sizeof(s_port), "%hu", port);
+		wget_snprintf(s_port, sizeof(s_port), "%hu", port);
 		debug_printf("resolving %s:%s...\n", host ? host : "", s_port);
 		return getaddrinfo(host, s_port, &hints, out_addr);
 	} else {
@@ -414,7 +407,7 @@ struct addrinfo *wget_tcp_resolve(wget_tcp_t *tcp, const char *host, uint16_t po
 
 		if (stats_callback) {
 			stats.ip = NULL;
-			stats_callback(WGET_STATS_TYPE_DNS, &stats);
+			stats_callback(&stats);
 		}
 
 		return NULL;
@@ -429,7 +422,7 @@ struct addrinfo *wget_tcp_resolve(wget_tcp_t *tcp, const char *host, uint16_t po
 		else
 			stats.ip = "???";
 
-		stats_callback(WGET_STATS_TYPE_DNS, &stats);
+		stats_callback(&stats);
 	}
 
 	/* Finally, print the address list to the debug pipe if enabled */
@@ -1011,7 +1004,7 @@ int wget_tcp_connect(wget_tcp_t *tcp, const char *host, uint16_t port)
 	int debug = wget_logger_is_active(wget_get_logger(WGET_LOGGER_DEBUG));
 
 	if (unlikely(!tcp))
-		return -1;
+		return WGET_E_INVALID;
 
 	if (tcp->addrinfo_allocated)
 		freeaddrinfo(tcp->addrinfo);
@@ -1054,7 +1047,7 @@ int wget_tcp_connect(wget_tcp_t *tcp, const char *host, uint16_t port)
 					error_printf(_("Failed to bind (%d)\n"), errno);
 					close(sockfd);
 
-					return -1;
+					return WGET_E_UNKNOWN;
 				}
 			}
 
@@ -1135,7 +1128,7 @@ int wget_tcp_tls_start(wget_tcp_t *tcp)
 
 /**
  * \param[in] tcp An active connection.
- * \return WGET_E_SUCCESS (0) on success, or a negative integer on error (one of WGET_E_XXX, defined in `<wget.h>`).
+ *
  * Stops TLS, but does not close the connection. Data will be transmitted in the clear from now on.
  */
 void wget_tcp_tls_stop(wget_tcp_t *tcp)
