@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012-2015 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -47,6 +47,7 @@
 #include <string.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
@@ -72,14 +73,14 @@ typedef struct
 {
 	const char
 		*hostname,
-		*version,
 		*alpn_protocol;
 	long long
 		tls_secs; //milliseconds
 	int
+		version,
 		cert_chain_size;
 	char
-		tcp_protocol,
+		http_protocol,
 		false_start,
 		tfo;
 	bool
@@ -94,13 +95,13 @@ typedef struct
 	int
 		nvalid,
 		nrevoked,
-		nignored;
+		nignored,
+		stapling;
 } _ocsp_stats_data_t;
 
-static wget_stats_callback_t stats_callback;
-static bool
-	ocsp_stats,
-	tls_stats;
+static wget_stats_callback_t
+	stats_callback_tls,
+	stats_callback_ocsp;
 
 static struct _config {
 	const char
@@ -1131,14 +1132,15 @@ static int _verify_certificate_callback(gnutls_session_t session)
 	}
 
 #ifdef HAVE_GNUTLS_OCSP_H
-	if (_config.ocsp && ocsp_stats) {
+	if (_config.ocsp && stats_callback_ocsp) {
 		_ocsp_stats_data_t stats;
 		stats.hostname = hostname;
 		stats.nvalid = nvalid;
 		stats.nrevoked = nrevoked;
 		stats.nignored = nignored;
+		stats.stapling = ctx->ocsp_stapling;
 
-		stats_callback(WGET_STATS_TYPE_OCSP, &stats);
+		stats_callback_ocsp(&stats);
 	}
 
 	if (_config.ocsp_stapling || _config.ocsp) {
@@ -1182,7 +1184,7 @@ static void __attribute__ ((destructor)) _wget_tls_exit(void)
 		wget_thread_mutex_destroy(&_mutex);
 }
 
-static _GL_INLINE int _key_type(int type)
+static int _key_type(int type)
 {
 	if (type == WGET_SSL_X509_FMT_DER)
 		return GNUTLS_X509_FMT_DER;
@@ -1280,7 +1282,7 @@ void wget_ssl_init(void)
 							struct stat st;
 							char fname[dirlen + 1 + len + 1];
 
-							snprintf(fname, sizeof(fname), "%s/%s", _config.ca_directory, dp->d_name);
+							wget_snprintf(fname, sizeof(fname), "%s/%s", _config.ca_directory, dp->d_name);
 							if (stat(fname, &st) == 0 && S_ISREG(st.st_mode)) {
 								debug_printf("GnuTLS loading %s\n", fname);
 								if ((rc = gnutls_certificate_set_x509_trust_file(_credentials, fname, GNUTLS_X509_FMT_PEM)) <= 0)
@@ -1399,11 +1401,30 @@ static int _do_handshake(gnutls_session_t session, int sockfd, int timeout)
 		}
 
 		if (gnutls_error_is_fatal(rc)) {
-			debug_printf("gnutls_handshake: (%d) %s\n", rc, gnutls_strerror(rc));
+			debug_printf("gnutls_handshake: (%d) %s (errno=%d)\n", rc, gnutls_strerror(rc),errno);
 
-			if (rc == GNUTLS_E_CERTIFICATE_ERROR)
+			if (rc == GNUTLS_E_CERTIFICATE_ERROR) {
 				ret = WGET_E_CERTIFICATE;
-			else
+			} else if (rc == GNUTLS_E_PUSH_ERROR && (errno == ECONNREFUSED || errno == ENOTCONN)) {
+				/*
+				 * ECONNREFUSED: on Linux
+				 * ENOTCONN: MinGW (in out Gitlab CI runner)
+				 */
+				ret = WGET_E_CONNECT;
+			} else if (rc == GNUTLS_E_PULL_ERROR && errno == 61 /* ENODATA, but not on OSX/Travis ? */) {
+				// We see this with older versions of GnuTLS, e.g. on TravisCI. (Tim, 11.4.2018)
+				// It happens when trying to connect to a port without a listener
+				ret = WGET_E_CONNECT;
+#ifdef GNUTLS_E_PREMATURE_TERMINATION
+			} else if (rc == GNUTLS_E_PREMATURE_TERMINATION && errno == EAGAIN) {
+				// It happens when trying to connect to a closed port
+				ret = WGET_E_CONNECT;
+#endif
+			} else if (rc == GNUTLS_E_UNEXPECTED_PACKET_LENGTH && errno == EAGAIN) {
+				// We see this with older versions of GnuTLS, e.g. on TravisCI. (Tim, 11.4.2018)
+				// It happens when trying to connect to a port without a listener
+				ret = WGET_E_CONNECT;
+			} else
 				ret = WGET_E_HANDSHAKE;
 
 			break;
@@ -1521,12 +1542,12 @@ int wget_ssl_open(wget_tcp_t *tcp)
 {
 	gnutls_session_t session;
 	_stats_data_t stats = {
-			.version = NULL,
 			.alpn_protocol = NULL,
+			.version = -1,
 			.false_start = -1,
 			.tfo = -1,
 			.resumed = 0,
-			.tcp_protocol = WGET_PROTOCOL_HTTP_1_1,
+			.http_protocol = WGET_PROTOCOL_HTTP_1_1,
 			.cert_chain_size = 0
 	};
 
@@ -1626,7 +1647,7 @@ int wget_ssl_open(wget_tcp_t *tcp)
 
 #ifdef MSG_FASTOPEN
 	if ((rc = wget_tcp_get_tcp_fastopen(tcp))) {
-		if (tls_stats)
+		if (stats_callback_tls)
 			stats.tfo = (char)rc;
 
 		// prepare for TCP FASTOPEN... sendmsg() instead of connect/write on first write
@@ -1663,17 +1684,17 @@ int wget_ssl_open(wget_tcp_t *tcp)
 		}
 	}
 
-	if (tls_stats)
+	if (stats_callback_tls)
 		before_millisecs = wget_get_timemillis();
 
 	ret = _do_handshake(session, sockfd, connect_timeout);
 
-	if (tls_stats) {
+	if (stats_callback_tls) {
 		long long after_millisecs = wget_get_timemillis();
 		stats.tls_secs = after_millisecs - before_millisecs;
 		stats.tls_con = 1;
 #if GNUTLS_VERSION_NUMBER >= 0x030500
-		stats.false_start = gnutls_session_get_flags(session) & GNUTLS_SFLAGS_FALSE_START;
+		stats.false_start = !! (gnutls_session_get_flags(session) & GNUTLS_SFLAGS_FALSE_START);
 #endif
 	}
 
@@ -1684,13 +1705,13 @@ int wget_ssl_open(wget_tcp_t *tcp)
 			debug_printf("GnuTLS: Get ALPN: %s\n", gnutls_strerror(rc));
 		else {
 			debug_printf("ALPN: Server accepted protocol '%.*s'\n", (int) protocol.size, protocol.data);
-			if (tls_stats)
+			if (stats_callback_tls)
 				stats.alpn_protocol = wget_strmemdup(protocol.data, protocol.size);
 
 			if (!memcmp(protocol.data, "h2", 2)) {
 				tcp->protocol = WGET_PROTOCOL_HTTP_2_0;
-				if (tls_stats)
-					stats.tcp_protocol = WGET_PROTOCOL_HTTP_2_0;
+				if (stats_callback_tls)
+					stats.http_protocol = WGET_PROTOCOL_HTTP_2_0;
 			}
 		}
 	}
@@ -1702,9 +1723,9 @@ int wget_ssl_open(wget_tcp_t *tcp)
 	if (ret == WGET_E_SUCCESS) {
 		int resumed = gnutls_session_is_resumed(session);
 
-		if (tls_stats) {
+		if (stats_callback_tls) {
 			stats.resumed = resumed;
-			stats.version = gnutls_protocol_get_name(gnutls_protocol_get_version(session));
+			stats.version = gnutls_protocol_get_version(session);
 			gnutls_certificate_get_peers(session, (unsigned int *)&(stats.cert_chain_size));
 		}
 
@@ -1726,9 +1747,9 @@ int wget_ssl_open(wget_tcp_t *tcp)
 		}
 	}
 
-	if (tls_stats) {
+	if (stats_callback_tls) {
 		stats.hostname = hostname;
-		stats_callback(WGET_STATS_TYPE_TLS, &stats);
+		stats_callback_tls(&stats);
 		xfree(stats.alpn_protocol);
 	}
 
@@ -1928,8 +1949,7 @@ ssize_t wget_ssl_write_timeout(void *session, const char *buf, size_t count, int
  */
 void wget_tcp_set_stats_tls(wget_stats_callback_t fn)
 {
-	stats_callback = fn;
-	tls_stats = (stats_callback != NULL);
+	stats_callback_tls = fn;
 }
 
 /**
@@ -1947,7 +1967,7 @@ const void *wget_tcp_get_stats_tls(wget_tls_stats_t type, const void *_stats)
 	case WGET_STATS_TLS_HOSTNAME:
 		return stats->hostname;
 	case WGET_STATS_TLS_VERSION:
-		return stats->version;
+		return &stats->version;
 	case WGET_STATS_TLS_FALSE_START:
 		return &(stats->false_start);
 	case WGET_STATS_TLS_TFO:
@@ -1958,8 +1978,8 @@ const void *wget_tcp_get_stats_tls(wget_tls_stats_t type, const void *_stats)
 		return &(stats->tls_con);
 	case WGET_STATS_TLS_RESUMED:
 		return &(stats->resumed);
-	case WGET_STATS_TLS_TCP_PROTO:
-		return &(stats->tcp_protocol);
+	case WGET_STATS_TLS_HTTP_PROTO:
+		return &(stats->http_protocol);
 	case WGET_STATS_TLS_CERT_CHAIN_SIZE:
 		return &(stats->cert_chain_size);
 	case WGET_STATS_TLS_SECS:
@@ -1976,8 +1996,7 @@ const void *wget_tcp_get_stats_tls(wget_tls_stats_t type, const void *_stats)
  */
 void wget_tcp_set_stats_ocsp(wget_stats_callback_t fn)
 {
-	stats_callback = fn;
-	ocsp_stats = (stats_callback != NULL);
+	stats_callback_ocsp = fn;
 }
 
 /**
@@ -2000,6 +2019,8 @@ const void *wget_tcp_get_stats_ocsp(wget_ocsp_stats_t type, const void *_stats)
 		return &(stats->nrevoked);
 	case WGET_STATS_OCSP_IGNORED:
 		return &(stats->nignored);
+	case WGET_STATS_OCSP_STAPLING:
+		return &(stats->stapling);
 	default:
 		return NULL;
 	}
@@ -2022,10 +2043,6 @@ int wget_ssl_open(wget_tcp_t *tcp) { return WGET_E_TLS_DISABLED; }
 void wget_ssl_close(void **session) { }
 ssize_t wget_ssl_read_timeout(void *session, char *buf, size_t count, int timeout) { return 0; }
 ssize_t wget_ssl_write_timeout(void *session, const char *buf, size_t count, int timeout) { return 0; }
-void wget_ssl_server_init(void) { }
-void wget_ssl_server_deinit(void) { }
-int wget_ssl_server_open(wget_tcp_t *tcp) { return WGET_E_TLS_DISABLED; }
-void wget_ssl_server_close(void **session) { }
 void wget_tcp_set_stats_tls(const wget_stats_callback_t fn) { }
 const void *wget_tcp_get_stats_tls(const wget_tls_stats_t type, const void *stats) { return NULL;}
 void wget_tcp_set_stats_ocsp(const wget_stats_callback_t fn) { }
