@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2017 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -100,8 +100,6 @@ static void _free_host_entry(HOST *host)
 	if (host) {
 		host_queue_free(host);
 		wget_robots_free(&host->robots);
-		wget_hashmap_free(&host->host_docs);
-		wget_hashmap_free(&host->tree_docs);
 		wget_xfree(host);
 	}
 }
@@ -113,7 +111,6 @@ HOST *host_add(wget_iri_t *iri)
 	if (!hosts) {
 		hosts = wget_hashmap_create(16, (wget_hashmap_hash_t)_host_hash, (wget_hashmap_compare_t)_host_compare);
 		wget_hashmap_set_key_destructor(hosts, (wget_hashmap_key_destructor_t)_free_host_entry);
-		stats_set_hosts(hosts, hosts_mutex);
 	}
 
 	HOST *hostp = NULL, host = { .scheme = iri->scheme, .host = iri->host, .port = iri->port };
@@ -329,11 +326,11 @@ void host_add_job(HOST *host, const JOB *job)
  * This function creates a priority job for robots.txt.
  * This job has to be processed before any other job.
  */
-void host_add_robotstxt_job(HOST *host, wget_iri_t *iri)
+void host_add_robotstxt_job(HOST *host, wget_iri_t *iri, bool http_fallback)
 {
 	JOB *job;
 
-	job = job_init(NULL, iri);
+	job = job_init(NULL, iri, http_fallback);
 	job->host = host;
 	job->robotstxt = 1;
 	job->local_filename = get_local_filename(job->iri);
@@ -484,7 +481,6 @@ void host_queue_free(HOST *host)
 	wget_list_browse(host->queue, (wget_list_browse_t)_queue_free_func, NULL);
 	wget_list_free(&host->queue);
 	if (host->robot_job) {
-		wget_iri_free(&host->robot_job->iri);
 		job_free(host->robot_job);
 		xfree(host->robot_job);
 	}
