@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -64,11 +64,12 @@
 #include "wget_plugin.h"
 #include "wget_stats.h"
 #include "wget_testing.h"
+#include "wget_utils.h"
 #ifdef WITH_GPGME
 #  include "wget_gpgme.h"
 #endif
 
-static int
+static exit_status_t
 	exit_status;
 
 void set_exit_status(exit_status_t status)
@@ -78,13 +79,17 @@ void set_exit_status(exit_status_t status)
 	// - error code 1 is used directly by exit() (fatal errors)
 	// - error codes 2... : lower numbers preceed higher numbers
 	if (exit_status) {
-		if ((int) status < exit_status)
+		if (status < exit_status) {
+			debug_printf("%s(%d)\n", __func__, (int) status);
 			exit_status = status;
-	} else
+		}
+	} else {
+		debug_printf("%s(%d)\n", __func__, (int) status);
 		exit_status = status;
+	}
 }
 
-int get_exit_status(void)
+exit_status_t get_exit_status(void)
 {
 	return exit_status;
 }
@@ -93,14 +98,15 @@ int get_exit_status(void)
 // SECTION_END. This must always be the last element of this enum and exactly
 // one greater than the value of the element before it.
 typedef enum {
-	SECTION_STARTUP = 0,
-	SECTION_DOWNLOAD = 1,
-	SECTION_HTTP = 2,
-	SECTION_SSL = 3,
-	SECTION_DIRECTORY = 4,
+	SECTION_STARTUP,
+	SECTION_DOWNLOAD,
+	SECTION_HTTP,
+	SECTION_SSL,
+	SECTION_DIRECTORY,
 #ifdef WITH_GPGME
 	SECTION_GPG,
 #endif
+	SECTION_PLUGIN,
 	SECTION_END,
 } help_section_t;
 
@@ -213,6 +219,12 @@ static int print_version(G_GNUC_WGET_UNUSED option_t opt, G_GNUC_WGET_UNUSED con
 #else
 	" -http2"
 #endif
+
+#if defined WITH_GPGME
+	" +gpgme"
+#else
+	" -gpgme"
+#endif
 	);
 #endif // #ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 
@@ -220,24 +232,6 @@ static int print_version(G_GNUC_WGET_UNUSED option_t opt, G_GNUC_WGET_UNUSED con
 	return -1; // stop processing & exit
 }
 
-static char *_shell_expand(const char *str)
-{
-	char *expanded_str = NULL;
-
-	if (*str == '~') {
-		char *pathptr = strchrnul(str, '/');
-		expanded_str = wget_strnglob(str, pathptr - str, GLOB_TILDE|GLOB_ONLYDIR|GLOB_NOCHECK);
-	}
-
-	// Either the string does not start with a "~", or the glob expansion
-	// failed. In both cases, return the original string back
-	if (!expanded_str) {
-		expanded_str = wget_strdup(str);
-	}
-
-	return expanded_str;
-}
-
 static int parse_integer(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
 {
 	*((int *)opt->var) = val ? atoi(val) : 0;
@@ -245,6 +239,19 @@ static int parse_integer(option_t opt, const char *val, G_GNUC_WGET_UNUSED const
 	return 0;
 }
 
+static int parse_uint16(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
+{
+	int port = val ? atoi(val) : 0;
+
+	if (port >= 0 && port <= UINT16_MAX) {
+		*((uint16_t *)opt->var) = (uint16_t) port;
+		return 0;
+	}
+
+	error_printf(_("Value out of range (0-65535): %s\n"), val);
+	return -1;
+}
+
 static int parse_numbytes(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
 {
 	if (val) {
@@ -283,17 +290,20 @@ static int parse_numbytes(option_t opt, const char *val, G_GNUC_WGET_UNUSED cons
 static int parse_filename(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
 {
 	xfree(*((const char **)opt->var));
-	*((const char **)opt->var) = val ? _shell_expand(val) : NULL;
+	*((const char **)opt->var) = val ? shell_expand(val) : NULL;
 
 	debug_printf("Expanded value = %s\n", *(const char **)opt->var);
 	return 0;
 }
 
-static int parse_string(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
+static int parse_string(option_t opt, const char *val, const char invert)
 {
+	if (opt->args == -1 && !invert && !val)
+		val = ""; // needed for stats options
+
 	// the strdup'ed string will be released on program exit
 	xfree(*((const char **)opt->var));
-	*((const char **)opt->var) = val ? wget_strdup(val) : NULL;
+	*((const char **)opt->var) = wget_strdup(val);
 
 	return 0;
 }
@@ -336,7 +346,7 @@ static int parse_header(option_t opt, const char *val, G_GNUC_WGET_UNUSED const
 
 		if (!v) {
 			v = *((wget_vector_t **)opt->var) =
-				wget_vector_create(8, 4, (wget_vector_compare_t)compare_wget_http_param);
+				wget_vector_create(8, (wget_vector_compare_t)compare_wget_http_param);
 			wget_vector_set_destructor(v, (wget_vector_destructor_t)wget_http_free_param);
 		}
 
@@ -380,7 +390,7 @@ static int parse_stringlist_expand(option_t opt, const char *val, int expand, in
 		const char *s, *p;
 
 		if (!v)
-			v = *((wget_vector_t **)opt->var) = wget_vector_create(8, -2, (wget_vector_compare_t)strcmp);
+			v = *((wget_vector_t **)opt->var) = wget_vector_create(8, (wget_vector_compare_t)strcmp);
 
 		for (s = p = val; *p; s = p + 1) {
 			if ((p = strchrnul(s, ',')) != s) {
@@ -392,7 +402,7 @@ static int parse_stringlist_expand(option_t opt, const char *val, int expand, in
 				const char *fname = wget_strmemdup(s, p - s);
 
 				if (expand && *s == '~') {
-					wget_vector_add_noalloc(v, _shell_expand(fname));
+					wget_vector_add_noalloc(v, shell_expand(fname));
 					xfree(fname);
 				} else
 					wget_vector_add_noalloc(v, fname);
@@ -470,7 +480,7 @@ static int parse_taglist(option_t opt, const char *val, G_GNUC_WGET_UNUSED const
 		const char *s, *p;
 
 		if (!v) {
-			v = *((wget_vector_t **)opt->var) = wget_vector_create(8, -2, (wget_vector_compare_t)_compare_tag);
+			v = *((wget_vector_t **)opt->var) = wget_vector_create(8, (wget_vector_compare_t)_compare_tag);
 			wget_vector_set_destructor(v, (wget_vector_destructor_t)_free_tag);
 		}
 
@@ -581,8 +591,10 @@ static int G_GNUC_WGET_PURE G_GNUC_WGET_NONNULL((1)) parse_regex_type(option_t o
 	else if (!wget_strcasecmp_ascii(val, "pcre"))
 		*((char *)opt->var) = WGET_REGEX_TYPE_PCRE;
 #endif
-	else
-		error_printf_exit(_("Unsupported regex type '%s'\n"), val);
+	else {
+		error_printf(_("Unsupported regex type '%s'\n"), val);
+		return -1;
+	}
 
 	return 0;
 }
@@ -665,11 +677,11 @@ static int parse_n_option(G_GNUC_WGET_UNUSED option_t opt, const char *val, G_GN
 static int parse_prefer_family(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
 {
 	if (!val || !wget_strcasecmp_ascii(val, "none"))
-		*((char *)opt->var) = WGET_NET_FAMILY_ANY;
+		*((int *)opt->var) = WGET_NET_FAMILY_ANY;
 	else if (!wget_strcasecmp_ascii(val, "ipv4"))
-		*((char *)opt->var) = WGET_NET_FAMILY_IPV4;
+		*((int *)opt->var) = WGET_NET_FAMILY_IPV4;
 	else if (!wget_strcasecmp_ascii(val, "ipv6"))
-		*((char *)opt->var) = WGET_NET_FAMILY_IPV6;
+		*((int *)opt->var) = WGET_NET_FAMILY_IPV6;
 	else {
 		error_printf(_("Unknown address family '%s'\n"), val);
 		return -1;
@@ -678,55 +690,25 @@ static int parse_prefer_family(option_t opt, const char *val, G_GNUC_WGET_UNUSED
 	return 0;
 }
 
-static int parse_stats(option_t opt, const char *val, const char invert)
+static int parse_stats_all(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
 {
-	int status, format = WGET_STATS_FORMAT_HUMAN;
-	char *filename = NULL;
+	xfree(*((const char **)opt->var));
+	*((const char **)opt->var) = wget_strdup(val);
 
-	if (!val || !strcmp(val, "1") || !wget_strcasecmp_ascii(val, "y") || !wget_strcasecmp_ascii(val, "yes") || !wget_strcasecmp_ascii(val, "on"))
-		status = !invert;
-	else if (!*val || !strcmp(val, "0") || !wget_strcasecmp_ascii(val, "n") || !wget_strcasecmp_ascii(val, "no") || !wget_strcasecmp_ascii(val, "off"))
-		status = invert;
-	else {
-		status = !invert;
+	xfree(config.stats_dns);
+	config.stats_dns = wget_strdup(val);
 
-		char *p;
-		if ((p = strchr(val, ':'))) {
-			if (!wget_strncasecmp_ascii("human", val, p - val) || !wget_strncasecmp_ascii("h", val, p - val))
-				;//empty statement
-			else if (!wget_strncasecmp_ascii("csv", val, p - val))
-				format = WGET_STATS_FORMAT_CSV;
-			else if (!wget_strncasecmp_ascii("json", val, p - val))
-				format = WGET_STATS_FORMAT_JSON;
-			else if ((int) (ptrdiff_t)opt->var == WGET_STATS_TYPE_SITE && !wget_strncasecmp_ascii("tree", val, p - val))
-				format = WGET_STATS_FORMAT_TREE;
-			else {
-				error_printf(_("Unknown stats format\n"));
-				return -1;
-			}
+	xfree(config.stats_ocsp);
+	config.stats_ocsp = wget_strdup(val);
 
-			val = p + 1;
-		}
+	xfree(config.stats_server);
+	config.stats_server = wget_strdup(val);
 
-		if (val)
-			filename = _shell_expand(val);
-	}
+	xfree(config.stats_site);
+	config.stats_site = wget_strdup(val);
 
-	stats_set_option((int) (ptrdiff_t) opt->var, status, format, filename);
-
-	return 0;
-}
-
-static int parse_stats_all(option_t opt, const char *val, const char invert)
-{
-	int rc;
-
-	if ((rc = parse_bool(opt, "1", invert)) < 0)
-		return rc;
-
-	if (config.stats_all)
-		for (int it = 1; it <= 5; it++)	// Get rid of magic number
-			parse_stats(opt + it, val, invert);
+	xfree(config.stats_tls);
+	config.stats_tls = wget_strdup(val);
 
 	return 0;
 }
@@ -818,9 +800,9 @@ static int parse_local_db(option_t opt, const char *val, const char invert)
 static int parse_report_speed_type(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
 {
 	if (!wget_strcasecmp_ascii(val, "bytes"))
-		*((int *)opt->var) = WGET_REPORT_SPEED_BYTES;
+		*((char *)opt->var) = WGET_REPORT_SPEED_BYTES;
 	else if (!wget_strcasecmp_ascii(val, "bits"))
-		*((int *)opt->var) = WGET_REPORT_SPEED_BITS;
+		*((char *)opt->var) = WGET_REPORT_SPEED_BITS;
 	else if (!val[0]) {
 		error_printf(_("Missing required type specifier\n"));
 		return -1;
@@ -833,13 +815,111 @@ static int parse_report_speed_type(option_t opt, const char *val, G_GNUC_WGET_UN
 	return 0;
 }
 
+static int parse_https_enforce(option_t opt, const char *val, G_GNUC_WGET_UNUSED const char invert)
+{
+	if (!wget_strcasecmp_ascii(val, "hard"))
+		*((char *)opt->var) = WGET_HTTPS_ENFORCE_HARD;
+	else if (!wget_strcasecmp_ascii(val, "soft"))
+		*((char *)opt->var) = WGET_HTTPS_ENFORCE_SOFT;
+	else if (!wget_strcasecmp_ascii(val, "none"))
+		*((char *)opt->var) = WGET_HTTPS_ENFORCE_NONE;
+	else if (!val[0]) {
+		error_printf("Missing required type specifier\n");
+		return -1;
+	}
+	else {
+		error_printf("Invalid type specifier: %s\n", val);
+		return -1;
+	}
+
+	return 0;
+}
+
+static int parse_compression(option_t opt, const char *val, const char invert)
+{
+	wget_vector_t *v = *((wget_vector_t **)opt->var);
+
+	if (!val && invert) {   // --no-compression
+		// should free the previous TYPE list and clear config.compression_methods for override
+		if (v) {
+			wget_vector_free((wget_vector_t **)opt->var);
+			config.compression_methods[wget_content_encoding_max] = 0;
+		}
+
+		config.no_compression = true;
+		return 0;
+	} else if (val && !invert) {    // --compression=TYPE
+		int rc;
+
+		// should free the previous TYPE list and clear config.compression_methods for override
+		if (v) {
+			wget_vector_free((wget_vector_t **)opt->var);
+			config.compression_methods[wget_content_encoding_max] = 0;
+		}
+
+		if ((rc = parse_stringlist_expand(opt, val, 0, 16)))
+			return rc;
+
+		v = *((wget_vector_t **)opt->var);
+
+		config.no_compression = false;
+		long long methods_bits = 0;		// check duplication
+
+		for (int it = 0; it < wget_vector_size(v); it++) {
+			int not_built = 0;
+			wget_content_encoding_type_t type = wget_content_encoding_by_name(wget_vector_get(v, it));
+
+			if (type == wget_content_encoding_unknown) {
+				wget_error_printf(_("Compression type %s not supported\n"), wget_content_encoding_to_name(type));
+				return -1;
+			} else if (methods_bits & (1 << type)) {
+				wget_error_printf(_("Duplicate type %s"), wget_content_encoding_to_name(type));
+				return -1;
+			}
+
+#ifndef WITH_ZLIB
+			if (type == wget_content_encoding_gzip || type == wget_content_encoding_deflate)
+				not_built = 1;
+#endif
+#ifndef WITH_BZIP2
+			if (type == wget_content_encoding_bzip2)
+				not_built = 1;
+#endif
+#ifndef WITH_LZMA
+			if (type == wget_content_encoding_xz || type == wget_content_encoding_lzma)
+				not_built = 1;
+#endif
+#ifndef WITH_BROTLIDEC
+			if (type == wget_content_encoding_brotli)
+				not_built = 1;
+#endif
+
+			if (not_built) {
+				wget_error_printf(_("Lib for type %s not built"), wget_content_encoding_to_name(type));
+				return -1;
+			}
+
+			methods_bits |= (1 << type);
+			config.compression_methods[config.compression_methods[wget_content_encoding_max]++] = type;
+		}
+
+		return 0;
+	} else if (!val && !invert) {   // --compression (for override)
+		config.no_compression = false;
+		return 0;
+	}
+
+	return -1;
+}
+
+
 static int list_plugins(G_GNUC_WGET_UNUSED option_t opt,
 	G_GNUC_WGET_UNUSED const char *val, G_GNUC_WGET_UNUSED const char invert)
 {
 	if (! plugin_loading_enabled)
 		return 0;
 
-	wget_vector_t *v = wget_vector_create(16, -2, NULL);
+	wget_vector_t *v = wget_vector_create(16, NULL);
 	plugin_db_list(v);
 
 	int n_names = wget_vector_size(v);
@@ -901,14 +981,13 @@ struct config config = {
 #if defined WITH_LIBNGHTTP2
 	.http2 = 1,
 	.http2_request_window = 30,
-	.http1_request_window = 10,
 #endif
 	.ocsp = 1,
 	.ocsp_stapling = 1,
 	.netrc = 1,
 	.waitretry = 10 * 1000,
 #ifdef WITH_GPGME
-	.sig_ext = "sig",
+	.verify_save_failed = 0,
 #endif
 	.metalink = 1,
 	.tls_false_start = 1,
@@ -948,8 +1027,7 @@ static const struct optionw options[] = {
 	},
 	{ "append-output", &config.logfile_append, parse_string, 1, 'a',
 		SECTION_STARTUP,
-		{ "File where messages are appended to,\n",
-		  "'-' for STDOUT.\n"
+		{ "File where messages are appended to, '-' for STDOUT\n"
 		}
 	},
 	{ "ask-password", &config.askpass, parse_bool, -1, 0,
@@ -1034,16 +1112,20 @@ static const struct optionw options[] = {
 		{ "Enable file clobbering. (default: on)\n"
 		}
 	},
-	{ "config", &config.config_files, parse_filenames, 1, 0,
-		SECTION_STARTUP,
-		{ "Backward compatibility\n"
-		}
-	}, // for backward compatibility only
-	{ "config-file", &config.config_files, parse_filenames, 1, 0,
-		SECTION_STARTUP,
-		{ "Path to a wgetrc file.\n"
+	{
+		"compression", &config.compression, parse_compression, -1, 0,
+		SECTION_HTTP,
+		{ "Customize Accept-Encoding with\n",
+		   "identity, gzip, deflate, xz, lzma, br, bzip2\n",
+		   "and any combination of it\n",
+		   "no-compression means no Accept-Encoding\n"
 		}
 	},
+	{ "config", &config.config_files, parse_filenames, 1, 0,
+		SECTION_STARTUP,
+		{  "List of config files. (default: ~/.wget2rc)\n"
+		}
+	}, // for backward compatibility only
 	{ "connect-timeout", &config.connect_timeout, parse_timeout, 1, 0,
 		SECTION_DOWNLOAD,
 		{ "Connect timeout in seconds.\n"
@@ -1076,8 +1158,7 @@ static const struct optionw options[] = {
 		SECTION_HTTP,
 		{ "Load public suffixes from file. \n",
 		  "They prevent 'supercookie' vulnerabilities.\n",
-		  "Download the list with:\n",
-		  "wget -O suffixes.txt https://publicsuffix.org/list/effective_tld_names.dat\n"
+		  "See https://publicsuffix.org/ for details.\n",
 		}
 	},
 	{ "cookies", &config.cookies, parse_bool, -1, 0,
@@ -1111,6 +1192,16 @@ static const struct optionw options[] = {
 		{ "Print debugging messages.(default: off)\n"
 		}
 	},
+	{ "default-http-port", &config.default_http_port, parse_uint16, 1, 0,
+		SECTION_HTTP,
+		{ "Set default port for HTTP. (default: 80)\n"
+		}
+	},
+	{ "default-https-port", &config.default_https_port, parse_uint16, 1, 0,
+		SECTION_HTTP,
+		{ "Set default port for HTTPS. (default: 443)\n"
+		}
+	},
 	{ "default-page", &config.default_page, parse_string, 1, 0,
 		SECTION_HTTP,
 		{ "Default file name. (default: index.html)\n"
@@ -1170,8 +1261,8 @@ static const struct optionw options[] = {
 	},
 	{ "filter-urls", &config.filter_urls, parse_bool, 0, 0,
 		SECTION_DOWNLOAD,
-		{ "Apply the accept and reject filters on the URL before starting a download.\n",
-		  "(default: off)\n"
+		{ "Apply the accept and reject filters on the URL\n",
+                  "before starting a download. (default: off)\n"
 		}
 	},
 	{ "follow-tags", &config.follow_tags, parse_taglist, 1, 0,
@@ -1182,8 +1273,7 @@ static const struct optionw options[] = {
 	},
 	{ "force-atom", &config.force_atom, parse_bool, -1, 0,
 		SECTION_STARTUP,
-		{ "Treat input file as Atom Feed.\n",
-		  "(default: off) (NEW!)\n"
+		{ "Treat input file as Atom Feed. (default: off) (NEW!)\n"
 		}
 	},
 	{ "force-css", &config.force_css, parse_bool, -1, 0,
@@ -1204,8 +1294,7 @@ static const struct optionw options[] = {
 	},
 	{ "force-metalink", &config.force_metalink, parse_bool, -1, 0,
 		SECTION_STARTUP,
-		{ "Treat input file as Metalink.\n",
-		  "(default: off) (NEW!)\n"
+		{ "Treat input file as Metalink. (default: off) (NEW!)\n"
 		}
 	},
 	{ "force-progress", &config.force_progress, parse_bool, -1, 0,
@@ -1216,8 +1305,7 @@ static const struct optionw options[] = {
 	},
 	{ "force-rss", &config.force_rss, parse_bool, -1, 0,
 		SECTION_STARTUP,
-		{ "Treat input file as RSS Feed.\n",
-		  "(default: off) (NEW!)\n"
+		{ "Treat input file as RSS Feed. (default: off) (NEW!)\n"
 		}
 	},
 	{ "force-sitemap", &config.force_sitemap, parse_bool, -1, 0,
@@ -1234,8 +1322,8 @@ static const struct optionw options[] = {
 #ifdef WITH_GPGME
 	{ "gnupg-homedir", &config.gnupg_homedir, parse_filename, -1, 0,
 		SECTION_GPG,
-		{ "Specify a directory to use as the GnuPG home directory.\n",
-		  "(default: gnupg default homedir)\n"
+		{ "Specify a directory to use as the GnuPG home\n",
+		  " directory. (default: gnupg default homedir)\n"
 		}
 	},
 #endif
@@ -1331,6 +1419,20 @@ static const struct optionw options[] = {
 		{ "Use HTTP/2 protocol if possible. (default: on)\n"
 		}
 	},
+	{ "http2-request-window", &config.http2_request_window, parse_integer, 1, 0,
+		SECTION_DOWNLOAD,
+		{ "Max. number of parallel streams per HTTP/2\n",
+                  "connection. (default: 30)\n"
+		}
+	},
+	{ "https-enforce", &config.https_enforce, parse_https_enforce, 1, 0,
+		SECTION_SSL,
+		{ "Use secure HTTPS instead of HTTP. Legal types are\n",
+                  "'hard', 'soft' and 'none'.\n",
+                  "If --https-only is enabled,\n",
+                  "this option has no effect. (default: none)\n"
+		}
+	},
 	{ "https-only", &config.https_only, parse_bool, -1, 0,
 		SECTION_SSL,
 		{ "Do not follow non-secure URLs. (default: off).\n"
@@ -1391,7 +1493,7 @@ static const struct optionw options[] = {
 		}
 	},
 	{ "list-plugins", NULL, list_plugins, 0, 0,
-		SECTION_STARTUP,
+		SECTION_PLUGIN,
 		{ "Lists all the plugins in the plugin search paths.\n"
 		}
 	},
@@ -1411,7 +1513,7 @@ static const struct optionw options[] = {
 		}
 	},
 	{ "local-plugin", NULL, parse_plugin_local, 1, 0,
-		SECTION_STARTUP,
+		SECTION_PLUGIN,
 		{ "Loads a plugin with a given path.\n"
 		}
 	},
@@ -1503,25 +1605,26 @@ static const struct optionw options[] = {
 		}
 	},
 	{ "plugin", NULL, parse_plugin, 1, 0,
-		SECTION_STARTUP,
+		SECTION_PLUGIN,
 		{ "Load a plugin with a given name.\n"
 		}
 	},
 	{ "plugin-dirs", NULL, parse_plugin_dirs, 1, 0,
-		SECTION_STARTUP,
+		SECTION_PLUGIN,
 		{ "Specify alternative directories to look\n",
 		  "for plugins, separated by ','\n"
 		}
 	},
 	{ "plugin-help", NULL, print_plugin_help, 0, 0,
-		SECTION_STARTUP,
+		SECTION_PLUGIN,
 		{ "Print help message for all loaded plugins\n"
 		}
 	},
 	{ "plugin-opt", NULL, parse_plugin_option, 1, 0,
-		SECTION_STARTUP,
+		SECTION_PLUGIN,
 		{ "Forward an option to a loaded plugin.\n",
-		  "The option should be in format <plugin_name>.<option>[=value]\n"
+		  "The option should be in format:\n",
+		  "<plugin_name>.<option>[=value]\n"
 		}
 	},
 	{ "post-data", &config.post_data, parse_string, 1, 0,
@@ -1609,9 +1712,11 @@ static const struct optionw options[] = {
 	{ "regex-type", &config.regex_type, parse_regex_type, 1, 0,
 		SECTION_DOWNLOAD,
 #if defined WITH_LIBPCRE2 || defined WITH_LIBPCRE
-		{ "Regular expression type. Possible types are posix or pcre. (default: posix)\n" }
+		{ "Regular expression type. Possible types are\n",
+                  "posix or pcre. (default: posix)\n" }
 #else
-		{ "Regular expression type. This build only supports posix. (default: posix)\n" }
+		{ "Regular expression type. This build only supports\n",
+                  "posix. (default: posix)\n" }
 #endif
 	},
 	{ "reject", &config.reject_patterns, parse_stringlist, 1, 'R',
@@ -1625,8 +1730,6 @@ static const struct optionw options[] = {
 		{ "Regex matching rejected URLs.\n"
 		}
 	},
-
-
 	{ "remote-encoding", &config.remote_encoding, parse_string, 1, 0,
 		SECTION_DOWNLOAD,
 		{ "Character encoding of remote files\n",
@@ -1646,6 +1749,12 @@ static const struct optionw options[] = {
 		  "uppercase, none\n"
 		}
 	},
+	{ "retry-connrefused", &config.retry_connrefused, parse_bool, -1, 0,
+		SECTION_HTTP,
+		{ "Consider \"connection refused\" a transient error.\n",
+		  " (default: off)\n"
+		}
+	},
 	{ "robots", &config.robots, parse_bool, -1, 0,
 		SECTION_DOWNLOAD,
 		{ "Respect robots.txt standard for recursive\n",
@@ -1676,10 +1785,10 @@ static const struct optionw options[] = {
 		}
 	},
 #ifdef WITH_GPGME
-	{ "signature-extension", &config.sig_ext, parse_string, 1, 0,
+	{ "signature-extensions", &config.sig_ext, parse_stringlist, 1, 0,
 		SECTION_GPG,
-		{ "The extension of the signature file which should be downloaded\n",
-		  "(default: sig)\n"
+		{ "The extension of the signature file which should be\n",
+		  "downloaded. (default: sig)\n"
 		}
 	},
 #endif
@@ -1697,37 +1806,43 @@ static const struct optionw options[] = {
 	{ "stats-all", &config.stats_all, parse_stats_all, -1, 0,
 		SECTION_STARTUP,
 		{ "Print all stats (default: off)\n",
-		  "Additional format supported: --stats-all[=[FORMAT:]FILE]\n"
+		  "Additional format supported:\n",
+                  "--stats-all[=[FORMAT:]FILE]\n"
 		}
 	},
-	{ "stats-dns", (void *) WGET_STATS_TYPE_DNS, parse_stats, -1, 0,
+	{ "stats-dns", &config.stats_dns, parse_string, -1, 0,
 		SECTION_STARTUP,
 		{ "Print DNS stats. (default: off)\n",
-		  "Additional format supported: --stats-dns[=[FORMAT:]FILE]\n"
+		  "Additional format supported:\n",
+                  "--stats-dns[=[FORMAT:]FILE]\n"
 		}
 	},
-	{ "stats-ocsp", (void *) WGET_STATS_TYPE_OCSP, parse_stats, -1, 0,
+	{ "stats-ocsp", &config.stats_ocsp, parse_string, -1, 0,
 		SECTION_STARTUP,
 		{ "Print OCSP stats. (default: off)\n",
-		  "Additional format supported: --stats-ocsp[=[FORMAT:]FILE]\n"
+		  "Additional format supported:\n",
+                  "--stats-ocsp[=[FORMAT:]FILE]\n"
 		}
 	},
-	{ "stats-server", (void *) WGET_STATS_TYPE_SERVER, parse_stats, -1, 0,
+	{ "stats-server", &config.stats_server, parse_string, -1, 0,
 		SECTION_STARTUP,
 		{ "Print server stats. (default: off)\n",
-		  "Additional format supported: --stats-server[=[FORMAT:]FILE]\n"
+		  "Additional format supported:\n",
+                  "--stats-server[=[FORMAT:]FILE]\n"
 		}
 	},
-	{ "stats-site", (void *) WGET_STATS_TYPE_SITE, parse_stats, -1, 0,
+	{ "stats-site", &config.stats_site, parse_string, -1, 0,
 		SECTION_STARTUP,
 		{ "Print site stats. (default: off)\n",
-		  "Additional format supported: --stats-site[=[FORMAT:]FILE]\n"
+		  "Additional format supported:\n",
+                  "--stats-site[=[FORMAT:]FILE]\n"
 		}
 	},
-	{ "stats-tls", (void *) WGET_STATS_TYPE_TLS, parse_stats, -1, 0,
+	{ "stats-tls", &config.stats_tls, parse_string, -1, 0,
 		SECTION_STARTUP,
 		{ "Print TLS stats. (default: off)\n",
-		  "Additional format supported: --stats-tls[=[FORMAT:]FILE]\n"
+		  "Additional format supported:\n",
+                  "--stats-tls[=[FORMAT:]FILE]\n"
 		}
 	},
 	{ "strict-comments", &config.strict_comments, parse_bool, -1, 0,
@@ -1781,7 +1896,8 @@ static const struct optionw options[] = {
 	},
 	{ "use-askpass", &config.use_askpass_bin, parse_string, 1, 0,
 		SECTION_DOWNLOAD,
-		{ "Prompt for a user and password using the specified command.\n"
+		{ "Prompt for a user and password using\n",
+                  "the specified command.\n"
 		}
 	},
 	{ "use-server-timestamps", &config.use_server_timestamps, parse_bool, -1, 0,
@@ -1798,8 +1914,8 @@ static const struct optionw options[] = {
 	},
 	{ "user-agent", &config.user_agent, parse_string, 1, 'U',
 		SECTION_HTTP,
-		{ "Username for Authentication.\n",
-		  "(default: empty username)\n"
+		{ "HTTP User Agent string.\n",
+		  "(default: wget)\n"
 		}
 	},
 	{ "verbose", &config.verbose, parse_bool, -1, 'v',
@@ -1808,6 +1924,12 @@ static const struct optionw options[] = {
 		}
 	},
 #ifdef WITH_GPGME
+	{ "verify-save-failed", &config.verify_save_failed, parse_bool, -1, 0,
+		SECTION_GPG,
+		{ "Save target files even when their signatures fail\n",
+		  "GPG validation. (default: off)\n"
+		}
+	},
 	{ "verify-sig", &config.verify_sig, parse_bool, -1, 's',
 		SECTION_GPG,
 		{ "Download .sig file and verify. (default: off)\n"
@@ -1847,14 +1969,14 @@ static int print_help(G_GNUC_WGET_UNUSED option_t opt, G_GNUC_WGET_UNUSED const
 #else
 static inline void print_first(const char s, const char *l, const char *msg)
 {
-	if (strlen(l) > 16) {
-		printf("  %c%-4c  --%s\n",
+	if (strlen(l) > 18) {
+		printf("  %c%-2c --%s\n",
 			(s ? '-' : ' '),
 			(s ? s : ' '),
 			l);
-		printf("%29s%s", "", msg);
+		printf("%28s%s", "", msg);
 	} else
-		printf("  %c%-4c  --%-16.16s  %s",
+		printf("  %c%-2c --%-18.18s  %s",
 			(s ? '-' : ' '),
 			(s ? s : ' '),
 			l, msg);
@@ -1862,7 +1984,7 @@ static inline void print_first(const char s, const char *l, const char *msg)
 
 static inline void print_next(const char *msg)
 {
-	printf("%31s%s", "", msg);
+	printf("%30s%s", "", msg);
 }
 
 static int print_help(G_GNUC_WGET_UNUSED option_t opt, G_GNUC_WGET_UNUSED const char *val, G_GNUC_WGET_UNUSED const char invert)
@@ -1901,6 +2023,10 @@ static int print_help(G_GNUC_WGET_UNUSED option_t opt, G_GNUC_WGET_UNUSED const
 			break;
 #endif
 
+		case SECTION_PLUGIN:
+			printf("Plugin options:\n");
+			break;
+
 		case SECTION_END:
 			break;
 
@@ -1910,6 +2036,8 @@ static int print_help(G_GNUC_WGET_UNUSED option_t opt, G_GNUC_WGET_UNUSED const
 		}
 		for (unsigned it = 0; it < countof(options); it++) {
 			if (options[it].section == sect) {
+				if (options[it].short_name == 'n')
+					continue; // do not display this special compatibility catch -n* option
 				print_first(options[it].short_name,
 				            options[it].long_name,
 				            options[it].help_str[0]);
@@ -2489,7 +2617,7 @@ static int use_askpass(void)
 	if (run_use_askpass("Type username:", &config.username) < 0)
 		return -1;
 
-	snprintf(question, sizeof(question), "Type password for '%s':", config.username);
+	wget_snprintf(question, sizeof(question), "Type password for '%s':", config.username);
 	xfree(config.password);
 
 	/* Prompt for password */
@@ -2529,16 +2657,11 @@ int init(int argc, const char **argv)
 	config.secure_protocol = wget_strdup(config.secure_protocol);
 	config.ca_directory = wget_strdup(config.ca_directory);
 	config.default_page = wget_strdup(config.default_page);
-	config.domains = wget_vector_create(16, -2, (wget_vector_compare_t)strcmp);
-#ifdef WITH_GPGME
-	config.sig_ext = wget_strdup(config.sig_ext);
-#endif
-//	config.exclude_domains = wget_vector_create(16, -2, NULL);
-
+	config.domains = wget_vector_create(16, (wget_vector_compare_t)strcmp);
 
 	// create list of default config file names
 	const char *env;
-	config.config_files = wget_vector_create(8, -2, NULL);
+	config.config_files = wget_vector_create(8, NULL);
 	if ((env = getenv ("SYSTEM_WGET2RC")) && *env)
 		wget_vector_add_str(config.config_files, env);
 	if ((env = getenv ("WGET2RC")) && *env)
@@ -2547,7 +2670,7 @@ int init(int argc, const char **argv)
 		// we don't want to complain about missing home .wget2rc
 		const char *cfgfile = wget_aprintf("%s/.wget2rc", home_dir);
 		if (access(cfgfile, R_OK) == 0)
-			wget_vector_add_noalloc(config.config_files, cfgfile);
+			wget_vector_insert_noalloc(config.config_files, cfgfile, 0);
 		else
 			xfree(cfgfile);
 	}
@@ -2621,6 +2744,10 @@ int init(int argc, const char **argv)
 	if ((n = parse_command_line(argc, argv)) < 0)
 		return -1;
 
+	// parse and initialize stats options
+	if (stats_init())
+		return -1;
+
 	if (plugin_db_help_forwarded()) {
 		set_exit_status(WG_EXIT_STATUS_NO_ERROR);
 		return -1; // stop processing & exit
@@ -2641,6 +2768,15 @@ int init(int argc, const char **argv)
 
 	log_init();
 
+	if (config.https_only && config.https_enforce)
+		// disable https enforce if https-only is enabled
+		config.https_enforce = WGET_HTTPS_ENFORCE_NONE;
+
+	if (config.default_http_port)
+		wget_iri_set_defaultport("http", config.default_http_port);
+	if (config.default_https_port)
+		wget_iri_set_defaultport("https", config.default_https_port);
+
 	// check for correct settings
 	if (config.max_threads < 1 || (config.max_threads > 1 && config.chunk_size))
 		config.max_threads = 1;
@@ -2748,7 +2884,7 @@ int init(int argc, const char **argv)
 		config.http_proxy_password = wget_strdup(config.password);
 
 	if (config.auth_no_challenge) {
-		config.default_challenges = wget_vector_create(1, 1, NULL);
+		config.default_challenges = wget_vector_create(1, NULL);
 		wget_http_challenge_t basic;
 		memset(&basic, 0, sizeof(basic));
 		basic.auth_scheme = wget_strdup("basic");
@@ -2767,11 +2903,13 @@ int init(int argc, const char **argv)
 	if (config.verify_sig) {
 #ifdef WITH_GPGME
 		init_gpgme();
+		if (!config.sig_ext) {
+			config.sig_ext = wget_vector_create(1, (wget_vector_compare_t)strcmp);
+			wget_vector_add_str(config.sig_ext, "sig");
+		}
 #endif
 	}
 
-	config.stats_site = stats_is_enabled(WGET_STATS_TYPE_SITE);
-
 	if ((rc = wget_net_init())) {
 		wget_error_printf(_("Failed to init networking (%d)"), rc);
 		return -1;
@@ -2862,6 +3000,8 @@ void deinit(void)
 {
 	wget_global_deinit();
 
+	stats_exit();
+
 	wget_cookie_db_free(&config.cookie_db);
 	wget_hsts_db_free(&config.hsts_db);
 	wget_hpkp_db_free(&config.hpkp_db);
@@ -2907,17 +3047,20 @@ void deinit(void)
 	xfree(config.save_cookies);
 	xfree(config.secure_protocol);
 	xfree(config.tls_session_file);
-#ifdef WITH_GPGME
-	xfree(config.sig_ext);
-#endif
 	xfree(config.user_agent);
 	xfree(config.use_askpass_bin);
 	xfree(config.username);
 	xfree(config.gnupg_homedir);
-	wget_vector_free(&config.mime_types);
+	xfree(config.stats_all);
+	xfree(config.stats_dns);
+	xfree(config.stats_ocsp);
+	xfree(config.stats_server);
+	xfree(config.stats_site);
+	xfree(config.stats_tls);
 
 	wget_iri_free(&config.base);
 
+	wget_vector_free(&config.mime_types);
 	wget_vector_free(&config.domains);
 	wget_vector_free(&config.exclude_domains);
 	wget_vector_free(&config.follow_tags);
@@ -2927,7 +3070,10 @@ void deinit(void)
 	wget_vector_free(&config.headers);
 	wget_vector_free(&config.config_files);
 	wget_vector_free(&config.default_challenges);
-
+	wget_vector_free(&config.compression);
+#ifdef WITH_GPGME
+	wget_vector_free(&config.sig_ext);
+#endif
 	wget_http_set_http_proxy(NULL, NULL);
 	wget_http_set_https_proxy(NULL, NULL);
 	wget_http_set_no_proxy(NULL, NULL);
