@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2013-2014 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -62,7 +62,8 @@
 static int
 	http_server_port,
 	https_server_port,
-	keep_tmpfiles;
+	keep_tmpfiles,
+	reject_https_connection;
 static wget_vector_t
 	*request_urls;
 static wget_test_url_t
@@ -98,21 +99,16 @@ enum SERVER_MODE {
 
 static char *_scan_directory(const char* data)
 {
-	char *path = strchr(data, '/');
-	if (path != 0) {
-		return path;
-	}
-	else
-		return NULL;
+	return strchr(data, '/');
 }
 
 static char *_parse_hostname(const char* data)
 {
 	if (!wget_strncasecmp_ascii(data, "http://", 7)) {
-		char *path = strchr(data += 7, '/');
-		return path;
-	} else
-		return NULL;
+		return strchr(data += 7, '/');
+	}
+
+	return NULL;
 }
 
 static void _replace_space_with_plus(wget_buffer_t *buf, const char *data)
@@ -129,7 +125,7 @@ static int _print_query_string(
 {
 	struct query_string *query = cls;
 
-	if (key && query->it == 0) {
+	if (key && !query->it) {
 		wget_buffer_strcpy(query->params, "?");
 		_replace_space_with_plus(query->params, key);
 		if (value) {
@@ -137,7 +133,7 @@ static int _print_query_string(
 			_replace_space_with_plus(query->params, value);
 		}
 	}
-	if (key && query->it != 0) {
+	if (key && query->it) {
 		wget_buffer_strcat(query->params, "&");
 		_replace_space_with_plus(query->params, key);
 		if (value) {
@@ -147,7 +143,7 @@ static int _print_query_string(
 	}
 
 	query->it++;
-    return MHD_YES;
+	return MHD_YES;
 }
 
 static int _print_header_range(
@@ -219,6 +215,9 @@ static int _answer_to_connection(
 	size_t body_len;
 	char content_len[100], content_range[100];
 
+	// whether or not this connection is HTTPS
+	bool https = !!MHD_get_connection_info(connection, MHD_CONNECTION_INFO_PROTOCOL);
+
 	// get query string
 	query.params = wget_buffer_alloc(1024);
 	query.it = 0;
@@ -261,16 +260,21 @@ static int _answer_to_connection(
 		wget_buffer_strcat(url_full, "index.html");
 
 	// it1 = iteration for urls data
-	unsigned int it1, found = 0, chunked = 0;
-	for (it1 = 0; it1 < nurls; it1++) {
+	unsigned int found = 0, chunked = 0;
+	for (unsigned it1 = 0; it1 < nurls && !found; it1++) {
+		if (urls[it1].http_only && https)
+			continue;
+		if (urls[it1].https_only && !https)
+			continue;
+
 		// create default page for directory without index page
-		char *dir = _scan_directory(url_full->data + 1);
-		if (dir != 0 && !strcmp(dir, "/"))
+		const char *dir = _scan_directory(url_full->data + 1);
+		if (dir && !strcmp(dir, "/"))
 			wget_buffer_strcat(url_full, "index.html");
 
 		// create default page for hostname without index page
-		char *host = _parse_hostname(url_full->data);
-		if (host != 0 && !strcmp(host, "/"))
+		const char *host = _parse_hostname(url_full->data);
+		if (host && !strcmp(host, "/"))
 			wget_buffer_strcat(url_full, "index.html");
 
 		// convert remote url into escaped char for iri encoding
@@ -279,9 +283,52 @@ static int _answer_to_connection(
 		MHD_http_unescape(url_iri->data);
 
 		if (!strcmp(url_full->data, url_iri->data)) {
+			size_t body_length =
+				urls[it1].body_len ? urls[it1].body_len
+				: (urls[it1].body ? strlen(urls[it1].body) : 0);
+
+			// check request headers
+			int bad_request = 0;
+
+			for (unsigned it2 = 0; urls[it1].expected_req_headers[it2]; it2++) {
+				const char *header = urls[it1].expected_req_headers[it2];
+				if (header) {
+					const char *header_value = strchr(header, ':');
+					const char *header_key = wget_strmemdup(header, header_value - header);
+					const char *got_val = MHD_lookup_connection_value(connection, MHD_HEADER_KIND, header_key);
+					// 400 Bad Request
+					if (!got_val || strcmp(got_val, header_value + 2)) {
+						bad_request = 1;
+						wget_xfree(header_key);
+						break;
+					}
+					wget_xfree(header_key);
+				}
+			}
+
+			// check unexpected headers
+			for (unsigned it2 = 0; urls[it1].unexpected_req_headers[it2] && !bad_request; it2++) {
+				const char *header_key = urls[it1].unexpected_req_headers[it2];
+				const char *got_val = MHD_lookup_connection_value(connection, MHD_HEADER_KIND, header_key);
+				// 400 Bad Request
+				if (got_val) {
+					bad_request = 1;
+					break;
+				}
+			}
+
+			// return with "400 Bad Request"
+			if (bad_request) {
+				response = MHD_create_response_from_buffer(0, (void *) "", MHD_RESPMEM_PERSISTENT);
+				ret = MHD_queue_response(connection, MHD_HTTP_BAD_REQUEST, response);
+				wget_buffer_free(&url_iri);
+				found = 1;
+				break;
+			}
+
 			// chunked encoding
 			if (!wget_strcmp(urls[it1].name + 3, "bad.txt")) {
-				response = MHD_create_response_from_buffer(strlen(urls[it1].body),
+				response = MHD_create_response_from_buffer(body_length,
 					(void *) urls[it1].body, MHD_RESPMEM_MUST_COPY);
 				ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
 				MHD_add_response_header(response, "Transfer-Encoding", "chunked");
@@ -290,7 +337,7 @@ static int _answer_to_connection(
 				found = 1;
 				break;
 			}
-			for (int it2 = 0; urls[it1].headers[it2] != NULL; it2++) {
+			for (int it2 = 0; urls[it1].headers[it2]; it2++) {
 				const char *header = urls[it1].headers[it2];
 				if (header) {
 					const char *header_value = strchr(header, ':');
@@ -304,7 +351,7 @@ static int _answer_to_connection(
 				struct ResponseContentCallbackParam *callback_param = wget_malloc(sizeof(struct ResponseContentCallbackParam));
 
 				callback_param->response_data = urls[it1].body;
-				callback_param->response_size = strlen(urls[it1].body);
+				callback_param->response_size = body_length;
 
 				response = MHD_create_response_from_callback(MHD_SIZE_UNKNOWN,
 					1024, &_callback, callback_param, &_free_callback_param);
@@ -316,7 +363,7 @@ static int _answer_to_connection(
 
 			// 404 with non-empty "body"
 			if (!wget_strcmp(urls[it1].code, "404 Not exist")) {
-				response = MHD_create_response_from_buffer(strlen(urls[it1].body),
+				response = MHD_create_response_from_buffer(body_length,
 					(void *) urls[it1].body, MHD_RESPMEM_MUST_COPY);
 				ret = MHD_queue_response(connection, MHD_HTTP_NOT_FOUND, response);
 				wget_buffer_free(&url_iri);
@@ -330,7 +377,7 @@ static int _answer_to_connection(
 			{
 				response = MHD_create_response_from_buffer(0, (void *) "", MHD_RESPMEM_PERSISTENT);
 				// it2 = iteration for headers
-				for (unsigned int it2 = 0; urls[it1].headers[it2] != NULL; it2++) {
+				for (unsigned it2 = 0; urls[it1].headers[it2]; it2++) {
 					const char *header = urls[it1].headers[it2];
 					if (header) {
 						const char *header_value = strchr(header, ':');
@@ -403,10 +450,10 @@ static int _answer_to_connection(
 			}
 			else if (*header_range->data) {
 				if (!strcmp(to_bytes_string, "-"))
-					to_bytes = strlen(urls[it1].body) - 1;
+					to_bytes = body_length - 1;
 				body_len = to_bytes - from_bytes + 1;
 
-				if (from_bytes > to_bytes || from_bytes >= (int) strlen(urls[it1].body)) {
+				if (from_bytes > to_bytes || from_bytes >= (int) body_length) {
 					response = MHD_create_response_from_buffer(0, (void *) "", MHD_RESPMEM_PERSISTENT);
 					ret = MHD_queue_response(connection, MHD_HTTP_RANGE_NOT_SATISFIABLE, response);
 				} else {
@@ -420,8 +467,7 @@ static int _answer_to_connection(
 					ret = MHD_queue_response(connection, MHD_HTTP_PARTIAL_CONTENT, response);
 				}
 			} else {
-				size_t len = urls[it1].body_len ? urls[it1].body_len : strlen(urls[it1].body);
-				response = MHD_create_response_from_buffer(len, (void *) urls[it1].body, MHD_RESPMEM_MUST_COPY);
+				response = MHD_create_response_from_buffer(body_length, (void *) urls[it1].body, MHD_RESPMEM_MUST_COPY);
 				ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
 			}
 
@@ -439,7 +485,6 @@ static int _answer_to_connection(
 				}
 			}
 
-			it1 = (unsigned int)nurls;
 			found = 1;
 		}
 
@@ -470,6 +515,14 @@ static void _http_server_stop(void)
 	wget_xfree(cert_pem);
 }
 
+static int _check_to_accept(
+	G_GNUC_WGET_UNUSED void *cls,
+	G_GNUC_WGET_UNUSED const struct sockaddr *addr,
+	G_GNUC_WGET_UNUSED socklen_t addrlen)
+{
+	return reject_https_connection ? MHD_NO : MHD_YES;
+}
+
 static int _http_server_start(int SERVER_MODE)
 {
 	uint16_t port_num = 0;
@@ -481,6 +534,9 @@ static int _http_server_start(int SERVER_MODE)
 			port_num, NULL, NULL, &_answer_to_connection, NULL,
 			MHD_OPTION_DIGEST_AUTH_RANDOM, sizeof(rnd), rnd,
 			MHD_OPTION_NONCE_NC_SIZE, 300,
+#ifdef MHD_OPTION_STRICT_FOR_CLIENT
+			MHD_OPTION_STRICT_FOR_CLIENT, 1,
+#endif
 			MHD_OPTION_END);
 
 		if (!httpdaemon)
@@ -498,9 +554,12 @@ static int _http_server_start(int SERVER_MODE)
 		}
 
 		httpsdaemon = MHD_start_daemon(MHD_USE_SELECT_INTERNALLY | MHD_USE_TLS,
-			port_num, NULL, NULL, &_answer_to_connection, NULL,
+			port_num, _check_to_accept, NULL, _answer_to_connection, NULL,
 			MHD_OPTION_HTTPS_MEM_KEY, key_pem,
 			MHD_OPTION_HTTPS_MEM_CERT, cert_pem,
+#ifdef MHD_OPTION_STRICT_FOR_CLIENT
+			MHD_OPTION_STRICT_FOR_CLIENT, 1,
+#endif
 			MHD_OPTION_END);
 
 		if (!httpsdaemon) {
@@ -665,19 +724,19 @@ static char *_insert_ports(const char *src)
 	if (!src || (!strstr(src, "{{port}}") && !strstr(src, "{{sslport}}")))
 		return NULL;
 
-	size_t ret_len = strlen(src) + 1;
-	char *ret = wget_malloc(ret_len);
+	size_t srclen = strlen(src) + 1;
+	char *ret = wget_malloc(srclen);
 	char *dst = ret;
 
 	while (*src) {
 		if (*src == '{') {
 			if (!strncmp(src, "{{port}}", 8)) {
-				dst += snprintf(dst, ret_len - (dst - ret), "%d", http_server_port);
+				dst += snprintf(dst, srclen - (dst - ret), "%d", http_server_port);
 				src += 8;
 				continue;
 			}
 			else if (!strncmp(src, "{{sslport}}", 11)) {
-				dst += snprintf(dst, ret_len - (dst - ret), "%d", https_server_port);
+				dst += snprintf(dst, srclen - (dst - ret), "%d", https_server_port);
 				src += 11;
 				continue;
 			}
@@ -707,8 +766,11 @@ static void _write_msg(const char *msg, size_t len)
 void wget_test_start_server(int first_key, ...)
 {
 	int rc, key;
-	size_t it;
 	va_list args;
+	bool start_http = 1;
+#ifdef WITH_GNUTLS
+	bool start_https = 1;
+#endif
 
 	wget_global_init(
 		WGET_DEBUG_FUNC, _write_msg,
@@ -730,15 +792,24 @@ void wget_test_start_server(int first_key, ...)
 		case WGET_TEST_RESPONSE_CODE:
 			response_code = va_arg(args, const char *);
 			break;
-*/		case WGET_TEST_EXPECTED_REQUEST_HEADER:
-			break;
-		case WGET_TEST_RESPONSE_URLS:
+*/		case WGET_TEST_RESPONSE_URLS:
 			urls = va_arg(args, wget_test_url_t *);
 			nurls = va_arg(args, size_t);
 			break;
 		case WGET_TEST_SERVER_SEND_CONTENT_LENGTH:
 			server_send_content_length = !!va_arg(args, int);
 			break;
+		case WGET_TEST_HTTPS_ONLY:
+			start_http = 0;
+			break;
+		case WGET_TEST_HTTP_ONLY:
+#ifdef WITH_GNUTLS
+			start_https = 0;
+#endif
+			break;
+		case WGET_TEST_HTTPS_REJECT_CONNECTIONS:
+			reject_https_connection = 1;
+			break;
 		case WGET_TEST_FEATURE_MHD:
 			break;
 		case WGET_TEST_FEATURE_TLS:
@@ -778,19 +849,18 @@ void wget_test_start_server(int first_key, ...)
 	if (chdir(tmpdir) != 0)
 		wget_error_printf_exit(_("Failed to change to tmpdir (%d)\n"), errno);
 
-	// init server SSL layer (default cert and key file types are PEM)
-	wget_ssl_set_config_string(WGET_SSL_CA_FILE, SRCDIR "/certs/x509-ca-cert.pem");
-	wget_ssl_set_config_string(WGET_SSL_CERT_FILE, SRCDIR "/certs/x509-server-cert.pem");
-	wget_ssl_set_config_string(WGET_SSL_KEY_FILE, SRCDIR "/certs/x509-server-key.pem");
-
 	// start HTTP server
-	if ((rc = _http_server_start(HTTP_MODE)) != 0)
-		wget_error_printf_exit(_("Failed to start HTTP server, error %d\n"), rc);
+	if (start_http) {
+		if ((rc = _http_server_start(HTTP_MODE)) != 0)
+			wget_error_printf_exit(_("Failed to start HTTP server, error %d\n"), rc);
+	}
 
 #ifdef WITH_GNUTLS
 	// start HTTPS server
-	if ((rc = _http_server_start(HTTPS_MODE)) != 0)
-		wget_error_printf_exit(_("Failed to start HTTPS server, error %d\n"), rc);
+	if (start_https) {
+		if ((rc = _http_server_start(HTTPS_MODE)) != 0)
+			wget_error_printf_exit(_("Failed to start HTTPS server, error %d\n"), rc);
+	}
 #endif
 
 	// now replace {{port}} in the body by the actual server port
@@ -802,7 +872,7 @@ void wget_test_start_server(int first_key, ...)
 			url->body_alloc = 1;
 		}
 
-		for (it = 0; it < countof(url->headers) && url->headers[it]; it++) {
+		for (unsigned it = 0; it < countof(url->headers) && url->headers[it]; it++) {
 			p = _insert_ports(url->headers[it]);
 
 			if (p) {
@@ -891,9 +961,9 @@ void wget_test(int first_key, ...)
 		*request_url,
 		*options = "",
 #ifdef _WIN32
-		*executable = BUILDDIR "\\..\\src\\wget2_noinstall" EXEEXT " -d --no-config --no-local-db --max-threads=1 --prefer-family=ipv4 --no-proxy";
+		*executable = BUILDDIR "\\..\\src\\wget2_noinstall" EXEEXT " -d --no-config --no-local-db --max-threads=1 --prefer-family=ipv4 --no-proxy --timeout 10";
 #else
-		*executable = BUILDDIR "/../src/wget2_noinstall" EXEEXT " -d --no-config --no-local-db --max-threads=1 --prefer-family=ipv4 --no-proxy";
+		*executable = BUILDDIR "/../src/wget2_noinstall" EXEEXT " -d --no-config --no-local-db --max-threads=1 --prefer-family=ipv4 --no-proxy --timeout 10";
 #endif
 	const wget_test_file_t
 		*expected_files = NULL,
@@ -911,11 +981,13 @@ void wget_test(int first_key, ...)
 		args;
 	char
 		server_send_content_length_old = server_send_content_length;
+	bool
+		options_alloc = 0;
 
 	keep_tmpfiles = 0;
 
 	if (!request_urls)
-		request_urls = wget_vector_create(8,8,NULL);
+		request_urls = wget_vector_create(8, NULL);
 
 	va_start (args, first_key);
 	for (key = first_key; key; key = va_arg(args, int)) {
@@ -938,8 +1010,15 @@ void wget_test(int first_key, ...)
 			existing_files = va_arg(args, const wget_test_file_t *);
 			break;
 		case WGET_TEST_OPTIONS:
+		{
 			options = va_arg(args, const char *);
+			const char *tmp = _insert_ports(options);
+			if (tmp) {
+				options = tmp;
+				options_alloc = 1;
+			}
 			break;
+		}
 		case WGET_TEST_KEEP_TMPFILES:
 			keep_tmpfiles = va_arg(args, int);
 			break;
@@ -995,13 +1074,23 @@ void wget_test(int first_key, ...)
 		else
 			wget_buffer_printf(cmd, "%s %s", executable, options);
 	} else if (!strcmp(valgrind, "1")) {
-		wget_buffer_printf(cmd, "valgrind --error-exitcode=301 --leak-check=yes --show-reachable=yes --track-origins=yes --suppressions=" SRCDIR "/valgrind-suppressions %s %s", executable, options);
+		wget_buffer_printf(cmd, "valgrind --error-exitcode=301 --leak-check=yes --show-reachable=yes --track-origins=yes --child-silent-after-fork=yes --suppressions=" SRCDIR "/valgrind-suppressions %s %s", executable, options);
 	} else
 		wget_buffer_printf(cmd, "%s %s %s", valgrind, executable, options);
 
 	for (it = 0; it < (size_t)wget_vector_size(request_urls); it++) {
-		wget_buffer_printf_append(cmd, " \"http://localhost:%d/%s\"",
-			http_server_port, (char *)wget_vector_get(request_urls, it));
+		request_url = wget_vector_get(request_urls, it);
+
+		if (!wget_strncasecmp_ascii(request_url, "http://", 7)
+			|| !wget_strncasecmp_ascii(request_url, "https://", 8))
+		{
+			char *tmp = _insert_ports(request_url);
+			wget_buffer_printf_append(cmd, " \"%s\"", tmp ? tmp : request_url);
+			wget_xfree(tmp);
+		} else {
+			wget_buffer_printf_append(cmd, " \"http://localhost:%d/%s\"",
+				http_server_port, request_url);
+		}
 	}
 	wget_buffer_strcat(cmd, " 2>&1");
 
@@ -1057,7 +1146,9 @@ void wget_test(int first_key, ...)
 						wget_error_printf_exit(_("Failed to read %lld bytes from file '%s/%s', just got %zd [%s]\n"),
 							(long long)st.st_size, tmpdir, fname, nbytes, options);
 
-					if (strlen(expected_files[it].content) != (size_t)nbytes || memcmp(expected_files[it].content, content, nbytes) != 0)
+					size_t content_length = expected_files[it].content_length ? expected_files[it].content_length : strlen(expected_files[it].content);
+
+					if (content_length != (size_t) nbytes || memcmp(expected_files[it].content, content, nbytes) != 0)
 						wget_error_printf_exit(_("Unexpected content in %s [%s]\n"), fname, options);
 				}
 
@@ -1075,6 +1166,9 @@ void wget_test(int first_key, ...)
 	wget_vector_clear(request_urls);
 	wget_buffer_free(&cmd);
 
+	if (options_alloc)
+		wget_xfree(options);
+
 	server_send_content_length = server_send_content_length_old;
 
 	// system("ls -la");
