@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2018 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -46,17 +46,17 @@ static int
 	ok,
 	failed;
 
-static void check(int result, const char *msg)
+static void check(int result, int line, const char *msg)
 {
 	if (result) {
 		ok++;
 	} else {
 		failed++;
-		wget_info_printf("%s\n", msg);
+		wget_info_printf("L%d: %s\n", line, msg);
 	}
 }
 
-#define CHECK(e) check(!!(e), #e)
+#define CHECK(e) check(!!(e), __LINE__, #e)
 
 static void test_mem(void)
 {
@@ -120,6 +120,28 @@ static void test_strlcpy(void)
 	CHECK(!strcmp(buf, "xxx"));
 }
 
+static void test_strscpy(void)
+{
+	char buf[4] = "x";
+
+	CHECK(wget_strscpy(NULL, "y", sizeof(buf)) == -1);
+	CHECK(!strcmp(buf, "x"));
+	CHECK(wget_strscpy(buf, "y", 0) == -1);
+	CHECK(!strcmp(buf, "x"));
+	CHECK(wget_strscpy(buf, NULL, 0) == -1);
+	CHECK(!strcmp(buf, "x"));
+	CHECK(wget_strscpy(buf, NULL, 5) == 0);
+	CHECK(!strcmp(buf, ""));
+	CHECK(wget_strscpy(buf, "x", sizeof(buf)) == 1);
+	CHECK(!strcmp(buf, "x"));
+	CHECK(wget_strscpy(buf, "", sizeof(buf)) == 0);
+	CHECK(!strcmp(buf, ""));
+	CHECK(wget_strscpy(buf, "xx", sizeof(buf)) == 2);
+	CHECK(!strcmp(buf, "xx"));
+	CHECK(wget_strscpy(buf, "xxxxx", sizeof(buf)) == 3);
+	CHECK(!strcmp(buf, "xxx"));
+}
+
 static void _test_buffer(wget_buffer_t *buf, const char *name)
 {
 	char test[256];
@@ -696,6 +718,7 @@ static void test_iri_relative_to_absolute(void)
 		{ H1, "..", H1"/" },
 		{ H1, "../", H1"/" },
 		{ H1, "foo", H1"/foo" },
+		{ H1, "foo%3A", H1"/foo%3A" }, // escaped colon
 		{ H1, "foo/bar", H1"/foo/bar" },
 		{ H1, "foo///bar", H1"/foo/bar" },
 		{ H1, "foo/.", H1"/foo/" },
@@ -1549,7 +1572,7 @@ static void test_parse_challenge(void)
 	wget_http_challenge_t *challenge;
 
 	// Testcases found here http://greenbytes.de/tech/tc/httpauth/
-	challenges = wget_vector_create(2, 2, NULL);
+	challenges = wget_vector_create(2, NULL);
 	wget_vector_set_destructor(challenges, (wget_vector_destructor_t)wget_http_free_challenge);
 
 	for (unsigned it = 0; it < countof(test_data); it++) {
@@ -1761,7 +1784,7 @@ static void test_vector(void)
 		txt_sorted[5] = { {""}, {"four"}, {"one"}, {"three"}, {"two"} },
 		*txt[countof(txt_sorted)];
 	wget_vector_t
-		*v = wget_vector_create(2, -2, (wget_vector_compare_t)compare_txt);
+		*v = wget_vector_create(2, (wget_vector_compare_t)compare_txt);
 	unsigned
 		it;
 	int
@@ -1915,7 +1938,7 @@ static void test_stringmap(void)
 	wget_http_free_challenge(&challenge);
 
 	wget_vector_t *challenges;
-	challenges = wget_vector_create(2, 2, NULL);
+	challenges = wget_vector_create(2, NULL);
 	wget_vector_set_destructor(challenges, (wget_vector_destructor_t)wget_http_free_challenge);
 	wget_http_parse_challenge("Basic realm=\"test realm\"", &challenge);
 	wget_vector_add(challenges, &challenge, sizeof(challenge));
@@ -2379,6 +2402,7 @@ int main(int argc, const char **argv)
 	// testing basic library functionality
 	test_mem();
 	test_strlcpy();
+	test_strscpy();
 	test_buffer();
 	test_buffer_printf();
 	test_utils();
